/*
	$Id: ArrayTypeTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Jari Hakkinen, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Platform;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>ArrayType</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate, platform, protocol, printed, printMapName, featureLock,
featureCount, blankFeatures</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>ArrayType is named ArrayDesign in BASE 2.</li>
</ul>
</dd>
</dl>

@author Gregory
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class ArrayTypeTransfer extends CommonItemTransfer
{
	/**
	 	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.ArrayTypeTransfer");
	
	private Platform generic = null;
	private UploadFileTransfer uploadFT = null;
	
	/**
	Constructs a new ArrayDesignTransfer
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param protocolT A finished {@link ProtocolTransfer}
	 */
	ArrayTypeTransfer(UserTransfer userT, GroupTransfer groupT,
			ProtocolTransfer protocolT, UploadFileTransfer uploadFT)
	{
		super(userT, groupT);
		this.uploadFT = uploadFT;
	}

	public void start()
	{
		log.info("Transferring ArrayTypes...");
		int count = getInt("countArrayTypes");
		initIdMap(count);
		startProgress(count, "ArrayTypes");
		PreparedStatement ps = prepareStatementFromFile("selectArrayTypes");
		runUnBatched(ps, 0, getBatchSize());
		done();
		log.info("Transferring ArrayTypes...DONE");
	}

	/**
	Creates an {@link ArrayDesign} from the current position in
	<code>ResultSet</code>.
	
	@param rs <code>ResultSet</code> containing the data for this item.
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected void createItem(ResultSet rs) throws BaseException, SQLException
	{
		DbControl dc = newDbControl();
		if (generic == null) generic = Platform.getByExternalId(dc, Platform.GENERIC);
		ArrayDesign arrayDesign = ArrayDesign.getNew(dc, generic);
		setCommonItemData(dc, (CommonItem) arrayDesign, rs);
		int printed = rs.getInt(11);
		String fileName = rs.getString(12);
		if (printed > 0)
		{
			File file = uploadFT.getPrintmapFileById(dc, rs.getInt(1));
			if (file != null)
			{
				DataFileType dft = null;
				if (printed == 1)
				{
					// printmap
					dft = DataFileType.getByExternalId(dc, DataFileType.GENERIC_PRINT_MAP);
				}
				else if (printed == 2)
				{
					// reporter map
					dft = DataFileType.getByExternalId(dc, DataFileType.GENERIC_REPORTER_MAP);
				}
				if (dft != null)
				{
					arrayDesign.getFileSet().setMember(file, dft);
				}
			}
		}
		dc.saveItem(arrayDesign);
		commit(dc);
		mapId(rs.getInt(1), arrayDesign.getId());
		if (log.isDebugEnabled())
		{
			log.debug("Transfered ArrayType: " + arrayDesign.getName() + "\tBASE1 id(" + rs.getInt(1));			
		}
	}

	/**
	Returns a ArrayDesign by the given base 1 array design(type) id.
	
	@param dbc <code>DbControl</code> used to get the ArrayDesign.
	@param base1ArrayDesignId
	@return <code>ArrayDesign</code> or null if the given scan id does not
	        exsist.
	 */
	public ArrayDesign getArrayDesignById(DbControl dbc, int base1ArrayDesignId)
	{
		try
		{
			Integer base2Id = getBase2Id(base1ArrayDesignId);
			return base2Id == null ? null : ArrayDesign.getById(dbc, base2Id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get ArrayDesign by id: '" + base1ArrayDesignId + "' in BASE1");
			if (log.isDebugEnabled())log.debug("\t" + e);
		}
		return null;
	}
}
