/*
	$Id: PlateTypeTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.PlateType;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

/**
This class provides methods to transfer PlateType data from base 1 to BASE 2.

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class PlateTypeTransfer extends CommonItemTransfer
{
	/**
	Number of wells to BASE 2 id map.
	 */
	private HashMap<Integer, PlateGeometry> geometry = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.PlateTypeTransfer");

	/**
	Constructs an PlateTypeTransfer
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	 */
	PlateTypeTransfer(UserTransfer userT, GroupTransfer groupT)
	{
		super(userT, groupT);
	}

	public void start()
	{
		log.info("Transferring PlateTypes...");
		int count = getInt("countPlateTypes");
		initIdMap(count);
		try
		{
			DbControl b2c = newDbControl();
			geometry = new HashMap<Integer, PlateGeometry>();
			
			ItemQuery<PlateGeometry> query = PlateGeometry.getQuery();
			for (PlateGeometry pg : query.list(b2c))
			{
				int size = pg.getRows() * pg.getColumns();
				geometry.put(size, pg);
			}
			b2c.close();
		}
		catch (BaseException e)
		{
			log.error("Transferring PlateTypes FAILED" , e);
			stop();
		}
		startProgress(count, "PlateTypes");
		run(prepareStatementFromFile("selectPlateTypes"), 0, getBatchSize());
		done();
		log.info("Transferring PlateTypes...DONE");
	}

	/**
	Creates an {@link PlateType} from the current position in
	<code>ResultSet</code>.
	
	@param b2c {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>PlateType</code>
	@throws BaseException If geometry doesn't exsist or something goes wrong
	        when getting a new PlateType.
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	PlateType createItem(DbControl b2c, ResultSet rs) throws BaseException,
			SQLException
	{
		PlateGeometry plateGeometry = geometry.get(new Integer(rs.getInt(9)));
		PlateType plateType = PlateType.getNew(b2c, plateGeometry);
		setCommonItemData(b2c, (CommonItem) plateType, rs);
		if (log.isDebugEnabled())
		{
			log.debug("Transfered plateType: " + plateType.getName());
			log.debug("--------------------------------------------------------------------------");
		}
		return plateType;
	}

	/**
	Returns a PlateType by the given base1PlateTypeId.
	
	@param dbc <code>DbControl</code> used to get the PlateType
	@param base1PlateTypeId
	@return PlateType or null if PlateType doesn't exist.
	 */
	public PlateType getPlateTypeById(DbControl dbc, int base1PlateTypeId)
	{
		try
		{
			Integer id = getBase2Id(base1PlateTypeId);
			return id == null ? null : PlateType.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get plate by id " + base1PlateTypeId + "from BASE1");
			if (log.isDebugEnabled()) log.debug("\t" + e);
		}
		return null;
	}
}
