/**
	$Id: Services.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.extensions.service;

import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.clients.web.extensions.Settings;
import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionsFilter;
import net.sf.basedb.util.extensions.ExtensionsInvoker;
import net.sf.basedb.util.extensions.Registry;
import net.sf.basedb.util.extensions.SingleExtensionFilter;
import net.sf.basedb.util.extensions.events.ExtensionPointEventFilter;

/**
	Utility class for starting and stopping services.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class Services
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.web.extensions.service.Services");

	public static final String EXTENSION_POINT_ID = "net.sf.basedb.clients.web.services";
	
	private static boolean initialised;
	private static Settings settings;
	
	/**
		Initialise and start the installed services. Called when the web server
		is started up.
	*/
	public static synchronized void init(Registry registry, Settings s)
	{
		if (initialised) return;
		log.info("Initialising services");
		settings = s;
		ExtensionsInvoker<ServiceControllerAction> invoker = getInvoker(null);
		invoker.render(new StartServiceRenderer());
		registry.registerEventHandler(
				new ServicesEventHandler(settings), 
				new ExtensionPointEventFilter(EXTENSION_POINT_ID));
		log.info("Initialising services done");
		initialised = true;
	}
	
	/**
		Stop all services.
		
	*/
	public static synchronized void close()
	{
		if (!initialised) return;
		log.info("Closing all services");
		ExtensionsInvoker<ServiceControllerAction> invoker = getInvoker(null);
		invoker.render(new StopServiceRenderer());
		settings = null;
		initialised = false;
		log.info("Closing all services done");
	}

	/**
		Start services as a result of an administrator has installed a new extension
		or updated/enabled an existing on.
		@param ext The extension, or null to start all enabled extension
	*/
	static synchronized void start(Extension ext)
	{
		if (!initialised) return;
		log.info("Starting services in extension: " + ext);
		ExtensionsInvoker<ServiceControllerAction> invoker = getInvoker(ext);
		invoker.render(new StartServiceRenderer());
		log.info("Starting services in extension done: " + ext);
	}
	
	/**
		Start services as a result of an administrator has uninstalled an extension
		or is updating/disabling an existing on.
		@param ext The extension, or null to stop all enabled extension
	*/
	static synchronized void stop(Extension ext)
	{
		if (!initialised) return;
		log.info("Stopping services in extension: " + ext);
		ExtensionsInvoker<ServiceControllerAction> invoker = getInvoker(ext);
		invoker.render(new StopServiceRenderer());
		log.info("Stopping services in extension done: " + ext);
	}

	@SuppressWarnings("unchecked")
	private static ExtensionsInvoker<ServiceControllerAction> getInvoker(Extension extension)
	{
		ExtensionsFilter filter = extension == null ?
				settings : new SingleExtensionFilter(extension.getId());
		return (ExtensionsInvoker<ServiceControllerAction>)ExtensionsControl.useExtensions(null, filter, EXTENSION_POINT_ID);
	}
	
}
