/*
 $Id: AbstractQuery.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 */
package net.sf.basedb.core;

import net.sf.basedb.core.query.AutoJoiner;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.QuerySection;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.core.query.Order;
import net.sf.basedb.core.query.Join;
import net.sf.basedb.util.NestedIterator;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.HashMap;
import java.util.Collection;
import java.util.Set;


/**
	An abstract implementation of the {@link Query} interface. This class
	implements all methods and also adds protected methods for generating the
	main and count queries. Implementors extending this class only needs
	to add methods for executing the query.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
abstract class AbstractQuery
	implements Query
{

	/**
		Log all SQL statements.
	*/
	private static final org.apache.log4j.Logger logSql = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.query.ql.AbstractQuery");

	/**
		So we don't always have to call logSql.debug()
	*/
	private static final boolean debugEnabled = logSql.isDebugEnabled();

	private boolean hasAutoJoined;
	private Map<Class<?>, AutoJoiner<?, ?>> autoJoiners;
	
	/**
		List of selections added to the query.
	*/
	private List<Select> selects;
	
	/**
		List of permanent selections added to the query.
	*/
	private List<Select> permanentSelects;
	
	/**
		Tempoarary selections, that are automatically cleared after each build.
	*/
	private List<Select> temporarySelects;
	
	/**
		The root enitity.
	*/
	private final String rootEntity;

	/**
		List of joins added to the query.
	*/
	private List<Join> joins;

	/**
		List of permanent joins added to the query.
	*/
	private List<Join> permanentJoins;

	/**
		Tempoarary joins, that are automatically cleared after each build.
	*/
	private List<Join> temporaryJoins;

		
	/**
		List of restrictions added to the query.
	*/
	private List<Restriction> restrictions;

	/**
		List of permanent restrictions added to the query.
	*/
	private List<Restriction> permanentRestrictions;

	/**
		List of group expressions added to the query.
	*/
	private List<Expression> groups;

	/**
		List of permanent group expressions added to the query.
	*/
	private List<Expression> permanentGroups;

	/**
		List of having restrictions added to the query.
	*/
	private List<Restriction> havings;

	/**
		List of permanent having restrictions added to the query.
	*/
	private List<Restriction> permanentHavings;

	/**
		List of orders added to the query.
	*/
	private List<Order> orders;
	
	/**
		List of permanent orders added to the query.
	*/
	private List<Order> permanentOrders;

	/**
		Parameters values added to the query.
	*/
	private Map<String, Object> parameters;

	/**
		Permanent parameter values added to the query.
	*/
	private Map<String, Object> permanentParameters;

	/**
		Maps parameters to types.
	*/
	private Map<String, Type> parameterTypes;
	
	/**
		The index of the first result that should be returned by the query.
	*/
	private int firstResult;

	/**
		The maximum number of items that should be returned by the query.
	*/
	private int maxResults;

	/**
		If the query should calculate the total number of matching items
		in the case that limits have been set with {@link #firstResult}
		or {@link #maxResults}.
	*/
	private boolean returnTotalCount;

	/**
		If the query should only return the same item once or not.
	*/
	private boolean distinct;

	/**
		Temporary list to hold theta join conditions if needed by the database.
	*/
	private List<String> thetaJoinConditions;
	
	/**
		If the {@link #count(DbControl)} method is executing or not.
	*/
	private boolean isCounting;

	/**
		The section of the query that is currently beeing built.
	*/
	private QuerySection querySection;
	
	/**
		Create a new query.
		@param rootEntity The root entity of the query
	*/
	AbstractQuery(String rootEntity)
	{
		this.rootEntity = rootEntity;
	}

	
	/*
		From the Query interface
		----------------------------------
	*/
	public void addAutoJoiner(AutoJoiner<?, ?> autoJoiner)
		throws InvalidDataException
	{
		if (autoJoiner == null) throw new InvalidUseOfNullException("autoJoiner");
		if (autoJoiners == null) autoJoiners = new HashMap<Class<?>, AutoJoiner<?, ?>>();
		autoJoiners.put(autoJoiner.getElementClass(), autoJoiner);
	}
	
	public void select(Select select)
		throws InvalidDataException, BaseException
	{
		if (select == null) throw new InvalidUseOfNullException("select");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more selections.");
		if (selects == null) selects = new LinkedList<Select>();
		selects.add(select);
	}
	
	public void selectPermanent(Select select)
		throws InvalidDataException, BaseException
	{
		if (select == null) throw new InvalidUseOfNullException("select");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more selections.");
		if (permanentSelects == null) permanentSelects = new LinkedList<Select>();
		permanentSelects.add(select);
	}
	
	public void join(Join join)
		throws InvalidDataException, BaseException
	{
		if (join == null) throw new InvalidUseOfNullException("join");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more joins.");
		if (joins == null) joins = new LinkedList<Join>();
		joins.add(join);
	}
	public void joinPermanent(Join join)
		throws InvalidDataException, BaseException
	{
		if (join == null) throw new InvalidUseOfNullException("join");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more joins.");
		if (permanentJoins == null) permanentJoins = new LinkedList<Join>();
		permanentJoins.add(join);
	}
	
	public void restrict(Restriction restriction)
		throws InvalidDataException, BaseException
	{
		if (restriction == null) throw new InvalidUseOfNullException("restriction");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more restrictions.");
		if (restrictions == null) restrictions = new LinkedList<Restriction>();
		restrictions.add(restriction);
	}
	public void restrictPermanent(Restriction restriction)
		throws InvalidDataException, BaseException
	{
		if (restriction == null) throw new InvalidUseOfNullException("restriction");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more restrictions.");
		if (permanentRestrictions == null) permanentRestrictions = new LinkedList<Restriction>();
		permanentRestrictions.add(restriction);
	} 

	public void group(Expression expression)
		throws InvalidDataException, BaseException
	{
		if (expression == null) throw new InvalidUseOfNullException("expression");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more groups.");
		if (groups == null) groups = new LinkedList<Expression>();
		groups.add(expression);
	}
	public void groupPermanent(Expression expression)
		throws InvalidDataException, BaseException
	{
		if (expression == null) throw new InvalidUseOfNullException("expression");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more groups.");
		if (permanentGroups == null) permanentGroups = new LinkedList<Expression>();
		permanentGroups.add(expression);
	} 

	public void having(Restriction restriction)
		throws InvalidDataException, BaseException
	{
		if (restriction == null) throw new InvalidUseOfNullException("restriction");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more restrictions.");
		if (havings == null) havings = new LinkedList<Restriction>();
		havings.add(restriction);
	}
	public void havingPermanent(Restriction restriction)
		throws InvalidDataException, BaseException
	{
		if (restriction == null) throw new InvalidUseOfNullException("restriction");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more restrictions.");
		if (permanentHavings == null) permanentHavings = new LinkedList<Restriction>();
		permanentHavings.add(restriction);
	} 

	public void order(Order order)
		throws InvalidDataException, BaseException
	{
		if (order == null) throw new InvalidUseOfNullException("order");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more orders.");
		if (orders == null) orders = new LinkedList<Order>();
		orders.add(order);
	}
	public void orderPermanent(Order order)
		throws InvalidDataException, BaseException
	{
		if (order == null) throw new InvalidUseOfNullException("order");
		if (isReadonly()) throw new BaseException("The query has already been executed. Use reset() before adding more orders.");
		if (permanentOrders == null) permanentOrders = new LinkedList<Order>();
		permanentOrders.add(order);
	}


	private void validateParameterType(String name, Object value, Type valueType)
	{
		if (value instanceof Collection)
		{
			Collection<?> c = (Collection<?>)value;
			int i = 0;
			for (Object o : c)
			{
				if (!valueType.isCorrectType(o))
				{
					throw new InvalidDataException("Incorrect parameter type for '" + 
						name + "[" + i + "]'. Expected " + valueType + "; found " + value.getClass().getName());
				}
				i++;
			}
		}
		else
		{
			if (!valueType.isCorrectType(value))
			{
				throw new InvalidDataException("Incorrect parameter type for '" + name +
					"'. Expected " + valueType + "; found " + value.getClass().getName());
			}
		}
	
	}
	
	public void setParameter(String name, Object value, Type valueType)
		throws InvalidDataException, BaseException
	{
		if (name == null) throw new InvalidUseOfNullException("name");
		if (value == null) throw new InvalidUseOfNullException("value");
		if (permanentParameters != null && permanentParameters.containsKey(name))
		{
			throw new InvalidDataException("The parameter "+name+" has already been permanently set and cannot be changed");
		}
		if (valueType != null)
		{
			if (value != null) validateParameterType(name, value, valueType);
			if (parameterTypes == null) parameterTypes = new HashMap<String, Type>();
			parameterTypes.put(name, valueType);
		}
		if (parameters == null) parameters = new HashMap<String, Object>();
		parameters.put(name, value);
	}
	public void setPermanentParameter(String name, Object value, Type valueType)
		throws InvalidDataException, BaseException
	{
		if (name == null) throw new InvalidUseOfNullException("name");
		if (value == null) throw new InvalidUseOfNullException("value");
		if (permanentParameters == null) permanentParameters = new HashMap<String, Object>();
		if (valueType != null)
		{
			if (value != null) validateParameterType(name, value, valueType);
			if (parameterTypes == null) parameterTypes = new HashMap<String, Type>();
			parameterTypes.put(name, valueType);
		}
		if (permanentParameters.containsKey(name))
		{
			throw new InvalidDataException("The parameter "+name+" has already been permanently set and cannot be changed");
		}
		permanentParameters.put(name, value);
	}

	public boolean hasParameterValue(String name)
	{
		return (parameters != null && parameters.containsKey(name)) || 
			(permanentParameters != null && permanentParameters.containsKey(name));
	}
	
	public void setFirstResult(int firstResult)
	{
		this.firstResult = firstResult;
	}
	
	public int getFirstResult()
	{
		return firstResult;
	}

	public void setMaxResults(int maxResults)
	{
		this.maxResults = maxResults;
	}
	
	public int getMaxResults()
	{
		return maxResults;
	}

	public void setReturnTotalCount(boolean flag)
	{
		this.returnTotalCount = flag;
	}
	
	public boolean isReturningTotalCount()
	{
		return returnTotalCount;
	}

	public void setDistinct(boolean flag)
	{
		this.distinct = flag;
	}
	
	public boolean isDistinct()
	{
		return distinct;
	}
	
	public boolean isReadonly()
	{
		return hasAutoJoined;
	}
	
	/**
		Resets the query, allowing you to add more restrictions, joins, order objects
		and parameters. Existing restrictions, joins and order objects are cleared.
		Permanent restrictions, orders and parameters cannot be cleared.
	*/
	public void reset()
	{
		if (selects != null) selects.clear();
		if (joins != null) joins.clear();
		if (restrictions != null) restrictions.clear();
		if (groups != null) groups.clear();
		if (havings != null) havings.clear();
		if (orders != null) orders.clear();
		if (parameters != null) parameters.clear();
		firstResult = 0;
		maxResults = 0;
		hasAutoJoined = false;
	}
	public boolean isCounting()
	{
		return isCounting;
	}
	public QuerySection getQuerySection()
	{
		return querySection;
	}
	// -------------------------------------------

	/*
		From the Object class
		----------------------------------
	*/
	public String toString()
	{
		return this.getClass().getSimpleName() + ": " + getMainQuery(null, !isReadonly());
	}
	// -------------------------------------------
	
	/**
		Set the counting status of the query.
	*/
	void setCounting(boolean isCounting)
	{
		this.isCounting = isCounting;
	}
	
	void temporarySelect(Select select)
		throws InvalidDataException, BaseException
	{
		if (temporarySelects == null) temporarySelects = new LinkedList<Select>();
		temporarySelects.add(select);
	}

	void temporaryJoin(Join join)
		throws InvalidDataException, BaseException
	{
		if (temporaryJoins == null) temporaryJoins = new LinkedList<Join>();
		temporaryJoins.add(join);
	}

	// TODO - add temporaryXxxx for other query elements
	
	void resetTemporary()
	{
		if (temporarySelects != null) temporarySelects.clear();
		if (temporaryJoins != null) temporaryJoins.clear();
		// TODO - more temporary query elements
	}
	
	/**
		Get all non-permanent selects.
	*/
	List<Select> getSelects()
	{
		return selects;
	}
	
	/**
		Get all permanent selects.
	*/
	List<Select> getPermanentSelects()
	{
		return permanentSelects;
	}
	
	// TODO - add getter method for rest of query elements
	
	/**
		Get all permanent parameter values.
	*/
	Map<String, Object> getPermanentParameters()
	{
		return permanentParameters;
	}
	
	/**
		Get all non-permanent parameter values.
	*/
	Map<String, Object> getParameters()
	{
		return parameters;
	}
	
	/**
		Get the types of all parameters.
	*/
	Map<String, Type> getParameterTypes()
	{
		return parameterTypes;
	}
	
	/**
		Get the value of the named parameter. This method first
		looks among the permanent parameters then among the non-permanent parameters.
		@param name The name of the parameter
		@return The value of the parameter, or null if not found
	*/
	Object getParameterValue(String name)
	{
		if (permanentParameters != null && permanentParameters.containsKey(name))
		{
			return permanentParameters.get(name);
		}
		else if (parameters != null)
		{
			return parameters.get(name);
		}
		return null;
	}

	/**
		Get the type of the parameter, or null if not known.
	*/
	Type getParameterType(String name)
	{
		return parameterTypes == null ? null : parameterTypes.get(name);
	}
	
	/**
		Build the main query string.
	*/
	String getMainQuery(DbControl dc, boolean autoJoin)
		throws BaseException
	{
		if (autoJoin) autoJoin();
		
		StringBuilder ql = new StringBuilder();
		ql.append(isDistinct() ? "SELECT DISTINCT " : "SELECT ");
		
		int appended = 0;
		final boolean useThetaJoin = HibernateUtil.useThetaJoin(getQueryType()); 
		// Append SELECT elements
		appended = appendSelects(ql, dc, temporarySelects, 0);
		appended += appendSelects(ql,dc, permanentSelects, appended);
		appended += appendSelects(ql,dc, selects, appended);
		
		querySection = QuerySection.FROM;
		ql.append(" FROM ").append(rootEntity).append(" ").append(getRootAlias());
		
		// Append JOIN elements
		thetaJoinConditions =  useThetaJoin ? new LinkedList<String>() : null;
		appended = appendJoins(ql, dc, temporaryJoins, useThetaJoin, 0);
		appended += appendJoins(ql, dc, permanentJoins, useThetaJoin, appended);
		appended += appendJoins(ql, dc, joins, useThetaJoin, appended);

		// Append WHERE elements
		appended = 0;
		if (thetaJoinConditions != null)
		{
			String deliminator = "WHERE ";
			for (String thetaJoinCondition : thetaJoinConditions)
			{
				ql.append(deliminator).append(thetaJoinCondition);
				appended++;
				deliminator = " AND ";
			}
		}
		
		// Append WHERE elements
		appended += appendRestrictions(ql, dc, permanentRestrictions, appended);
		appended += appendRestrictions(ql, dc, restrictions, appended);
		
		// Append GROUP BY elements
		appended = appendGroups(ql, dc, permanentGroups, 0);
		appended += appendGroups(ql, dc, groups, appended);
		
		// Append HAVING elements
		appended = appendHavings(ql, dc, permanentHavings, 0);
		appended += appendHavings(ql, dc, havings, appended);
		
		// Append ORDER BY elements
		appended = appendOrders(ql, dc, permanentOrders, 0);
		appended += appendOrders(ql, dc, orders, appended);
		
		if (debugEnabled) logSql.debug("Main query(" + getQueryType()+ "): " +ql.toString());
		return ql.toString();
	}
	
	/**
		Build the query string for returning the total count.
	*/
	String getCountQuery(DbControl dc, boolean autoJoin)
		throws BaseException
	{
		if (autoJoin) autoJoin();
		
		StringBuilder ql = new StringBuilder();
		setCounting(true);
		
		if (isDistinct())
		{
			ql.append("SELECT COUNT(DISTINCT ");
			int appended = 0;
			// Append SELECT elements
			appended = appendSelects(ql, dc, temporarySelects, 0);
			appended += appendSelects(ql,dc, permanentSelects, appended);
			appended += appendSelects(ql,dc, selects, appended);
			ql.append(") ");
		}
		else
		{
			ql.append("SELECT COUNT(*) ");
		}
		
		querySection = QuerySection.FROM;
		ql.append(" FROM ").append(rootEntity).append(" ").append(getRootAlias());
		
		int appended = 0;
		final boolean useThetaJoin = HibernateUtil.useThetaJoin(getQueryType()); 
		// Append JOIN elements
		thetaJoinConditions =  useThetaJoin ? new LinkedList<String>() : null;
		appended = appendJoins(ql, dc, permanentJoins, useThetaJoin, 0);
		appended += appendJoins(ql, dc, joins, useThetaJoin, appended);
		appended += appendJoins(ql, dc, temporaryJoins, useThetaJoin, appended);
		
		// Append WHERE elements
		appended = 0;
		if (thetaJoinConditions != null)
		{
			String deliminator = "WHERE ";
			for (String thetaJoinCondition : thetaJoinConditions)
			{
				ql.append(deliminator).append(thetaJoinCondition);
				appended++;
				deliminator = " AND ";
			}
		}
		appended += appendRestrictions(ql, dc, permanentRestrictions, appended);
		appended += appendRestrictions(ql, dc, restrictions, appended);
		
		// Append GROUP BY elements
		appended = appendGroups(ql, dc, permanentGroups, 0);
		appended += appendGroups(ql, dc, groups, appended);
		
		// Append HAVING elements
		appended = appendHavings(ql, dc, permanentHavings, 0);
		appended += appendHavings(ql, dc, havings, appended);
		setCounting(false);
		
		if (debugEnabled) logSql.debug("Count query(" + getQueryType()+ "): " +ql.toString());
		return ql.toString();
	}
	
	private int appendSelects(StringBuilder ql, DbControl dc, List<Select> selects, int alreadyAppended)
		throws BaseException
	{
		if (selects == null) return 0;
		String deliminator = alreadyAppended == 0 ? "" : ", ";
		int appended = 0;
		querySection = QuerySection.SELECT;
		for (Select s : selects)
		{
			if (!isCounting || s.useInDistinctCount())
			{
				ql.append(deliminator);
				ql.append(dc == null ? s.toString() : s.toQl(this, dc));
				appended++;
				deliminator = ", ";
			}
		}
		return appended;
	}

	private int appendJoins(StringBuilder ql, DbControl dc, List<Join> joins, boolean useThetaJoin, int alreadyAppended)
		throws BaseException
	{
		if (joins == null) return 0;
		int appended = 0;
		querySection = QuerySection.JOIN;
		for (Join j : joins)
		{
			if (useThetaJoin && dc != null)
			{
				ql.append(" ").append(j.getThetaJoin(this, dc)).append(" ");
				thetaJoinConditions.add(j.getThetaJoinCondition(this, dc));
			}
			else
			{
				ql.append(" ").append(dc == null ? j.toString() : j.toQl(this, dc)).append(" ");
			}
			appended++;
		}
		return appended;
	}
	
	private int appendRestrictions(StringBuilder ql, DbControl dc, List<Restriction> restrictions, int alreadyAppended)
		throws BaseException
	{
		if (restrictions == null) return 0;
		String deliminator = alreadyAppended == 0 ? " WHERE " : " AND ";
		int appended = 0;
		querySection = QuerySection.WHERE;
		for (Restriction r : restrictions)
		{
			ql.append(deliminator);
			ql.append(dc == null ? r.toString() : r.toQl(this, dc));
			deliminator = " AND ";
			appended++;
		}
		return appended;
	}

	private int appendGroups(StringBuilder ql, DbControl dc, List<Expression> groups, int alreadyAppended)
		throws BaseException
	{
		if (groups == null) return 0;
		String deliminator = alreadyAppended == 0 ? " GROUP BY " : ", ";
		int appended = 0;
		querySection = QuerySection.GROUPBY;
		for (Expression e : groups)
		{
			ql.append(deliminator);
			ql.append(dc == null ? e.toString() : e.toQl(this, dc));
			deliminator = ", ";
			appended++;
		}
		return appended;
	}

	private int appendHavings(StringBuilder ql, DbControl dc, List<Restriction> havings, int alreadyAppended)
		throws BaseException
	{
		if (havings == null) return 0;
		String deliminator = alreadyAppended == 0 ? " HAVING " : " AND ";
		int appended = 0;
		querySection = QuerySection.HAVING;
		for (Restriction r : havings)
		{
			ql.append(deliminator);
			ql.append(dc == null ? r.toString() : r.toQl(this, dc));
			deliminator = " AND ";
			appended++;
		}
		return appended;
	}

	private int appendOrders(StringBuilder ql, DbControl dc, List<Order> orders, int alreadyAppended)
		throws BaseException
	{
		if (orders == null) return 0;
		String deliminator = alreadyAppended == 0 ? " ORDER BY " : ", ";
		int appended = 0;
		querySection = QuerySection.ORDERBY;
		for (Order o : orders)
		{
			ql.append(deliminator);
			ql.append(dc == null ? o.toString() : o.toQl(this, dc));
			deliminator = ", ";
			appended++;
		}
		return appended;
	}

	
	@SuppressWarnings("unchecked")
	private void autoJoin()
	{
		if (autoJoiners == null || hasAutoJoined) return;
		
		Iterator<QueryElement> iterator = new NestedIterator<QueryElement>(
			selects, permanentSelects, temporarySelects,
			joins, permanentJoins, temporaryJoins,
			restrictions, permanentRestrictions,
			groups, permanentGroups,
			havings, permanentHavings,
			orders, permanentOrders
		);
		
		Set<QueryElement> all = new HashSet<QueryElement>();
		while (iterator.hasNext())
		{
			QueryElement element = iterator.next();
			all.add(element);
			addChildElements(all, element);
		}
		
		for (QueryElement element : all)
		{
			AutoJoiner joiner = autoJoiners.get(element.getClass());
			if (joiner != null)
			{
				joiner.joinIfNeeded(this, element);
			}
		}
		hasAutoJoined = true;
	}
	
	private void addChildElements(Set<QueryElement> all, QueryElement root)
	{
		Collection<? extends QueryElement> children = root.getChildren();
		if (children != null)
		{
			for (QueryElement child : children)
			{
				if (child != null && all.add(child)) addChildElements(all, child);
			}
		}
	}
	
}
