/*
	$Id: AbstractSqlQuery.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.SqlQuery;
import net.sf.basedb.core.query.QueryType;
import net.sf.basedb.core.query.Select;
import net.sf.basedb.core.signal.SignalException;

import java.util.List;
import java.util.LinkedList;
import java.util.Collection;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
	An abstract implementation of the {@link SqlQuery} interface. 

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
abstract class AbstractSqlQuery
	extends AbstractQuery
	implements SqlQuery
{
	/**
		Log all SQL statements.
	*/
	private static final org.apache.log4j.Logger logSql = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.query.ql");

	/**
		So we don't always have to call logSql.debug()
	*/
	private static final boolean debugSqlEnabled = logSql.isDebugEnabled();

	/**
		Log all parameter bindings to prepared statement.
	*/
	private static final org.apache.log4j.Logger logParam = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.query.bind");
	
	/**
		So we don't always have to call logParam.debug()
	*/
	private static final boolean debugParamEnabled = logParam.isDebugEnabled();
		
	/**
		Pattern that finds a named parameter in an SQL string. A named parameter
		is a colon (:) followed by one or more alphanumeric characters.
	*/
	private static final Pattern namedParameter = Pattern.compile(":(\\w+)");

	/**
	*/
	AbstractSqlQuery(String rootEntity)
	{
		super(rootEntity);
	}
	
	/*
		From the Query interface
		-------------------------------------------
	*/
	/**
		@return {@link QueryType#SQL}
	*/
	public QueryType getQueryType()
	{
		return QueryType.SQL;
	}
	public long count(DbControl dc)
		throws BaseException
	{
		int totalCount = -1;
		// Get query string
		String countSql = getCountQuery(dc, true);
		// Parse SQL and replace named parameters with ?
		List<String> countParameterOrder = new LinkedList<String>();
		countSql = parseParameters(countSql, countParameterOrder);
		try
		{
			Connection c = HibernateUtil.getConnection(dc.getHibernateSession());
			if (debugSqlEnabled) logSql.debug("Executing count query: " + countSql);
			final PreparedStatement ps = c.prepareStatement(countSql);
			setParameters(ps, countParameterOrder);
			
			ResultSet result = QueryExecutor.executeQuery(ps, dc.getSessionControl());

			result.next();
			totalCount = result.getInt(1);
			result.close();
			ps.close();
		}
		catch (InterruptedException ex)
		{
			throw new SignalException("Aborted by user", ex);
		}
		catch (SQLException ex)
		{
			logSql.error(countSql, ex);
			throw new BaseException(ex);
		}
		return totalCount;
	}
	// -------------------------------------------
	
	abstract Select[] getDefaultSelects();

	/**
		Execute the query.
		@param dc The <code>DbControl</code> to use for database access
		@return A <code>DynamicResultIterator</code> object
		@throws BaseException If there is an error
	*/
	public DynamicResultIterator iterate(DbControl dc)
		throws BaseException
	{
		long totalCount = -1;

		// Reset temporary query elements
		resetTemporary();

		// If no selections has been specified, we use the defaults...
		if ((getSelects() == null || getSelects().size() == 0) && 
			(getPermanentSelects() == null || getPermanentSelects().size() == 0))
		{
			Select[] defaults = getDefaultSelects();
			if (defaults != null)
			{
				for (Select s : defaults)
				{
					if (s != null) temporarySelect(s);
				}
			}
		}
		
		// Get the main query
		String sql = getMainQuery(dc, true);
		
		// Holds the names of the parameters in the query in the order they appear
		List<String> parameterOrder = new LinkedList<String>();
		
		// Parse out named parameters
		sql = parseParameters(sql, parameterOrder);
		
		// Add limit parameters
		sql = bindLimits(sql, parameterOrder);
		// Main query is now built
		
		// Load the total count if it is requested
		if (isReturningTotalCount())
		{
			totalCount = count(dc);
		}

		ResultSet result = null;
		try
		{
			Connection c = HibernateUtil.getConnection(dc.getHibernateSession());
			if (debugSqlEnabled) logSql.debug("Executing main query: " + sql);
			PreparedStatement ps = c.prepareStatement(sql);
			setParameters(ps, parameterOrder);
			result = QueryExecutor.executeQuery(ps, dc.getSessionControl());
			if ((getFirstResult() > 0) && !HibernateUtil.getDialect().supportsLimitOffset())
			{
				result.absolute(getFirstResult());
			}
		}
		catch (InterruptedException ex)
		{
			throw new SignalException("Aborted by user.", ex);
		}
		catch (SQLException ex)
		{
			logSql.error(sql, ex);
			throw new BaseException(ex);
		}
		return new DynamicResultIterator(result, totalCount, getMaxResults());
	}

	/**
		Parse the given SQL and find named parameters. A named parameter
		is a colon (:) followed by one or more alphanumeric characters.
		Replaces the named parameters with a JDBC parameter placeholder,
		ie. a question mark. The name of the parameters are put in the
		specified list in the order they appear in the SQL string.
		@param sql The original SQL string, which may contain named
			parameters
		@param parameterOrder A list that should store the name of the
			parameters in the correct order
		@return The new SQL statement with JDBC question marks instead
			of named parameters
	*/
	String parseParameters(String sql, List<String> parameterOrder)
	{
		Matcher m = namedParameter.matcher(sql);
		StringBuffer sb = new StringBuffer();
		while (m.find())
		{
			String name = m.group(1);
			if (parameterOrder != null) parameterOrder.add(name);
			Object parameterValue = getParameterValue(name);
			if (parameterValue instanceof Collection)
			{
				// If the parameter is a collection one question mark is required for each value
				int size = ((Collection)parameterValue).size();
				StringBuilder sb2 = new StringBuilder();
				for (int i = 0; i < size; ++i)
				{
					if (i > 0) sb2.append(",");
					sb2.append("?");
				}
				m.appendReplacement(sb, sb2.toString());
			}
			else
			{
				m.appendReplacement(sb, "?");
			}
		}
		m.appendTail(sb);
		return sb.toString();
	}

	/**
		Parse the given SQL and add LIMIT and OFFSET parameters to it. This method
		checks the capabilities of the installed Hibernate <code>Dialect</code>
		and uses it for generating the actual SQL. If limit parameters are added
		to the query this method adds entries for those parameters in the 
		<code>parameterOrder</code> list and values using the regular
		{@link net.sf.basedb.core.query.Query#setParameter(String, Object, Type)} method. 
		The parameter names used are <code>_limit_</code> and <code>_offset_</code>.
		<p>
		If the underlying database doesn't support limit parameters the query
		will be executed and the result starting point moved forward to the
		correct offset and the a limit will be passed to the {@link DynamicResultIterator}
		object.

		@param sql The original SQL without limit parameters
		@param parameterOrder A list that stores the names of the parameters
			in the order they should be binf by JDBC
		@return The new SQL with limit parameters
	*/
	String bindLimits(String sql, List<String> parameterOrder)
	{
		org.hibernate.dialect.Dialect dialect = HibernateUtil.getDialect();
		boolean hasLimit = getMaxResults() > 0;
		boolean hasOffset = getFirstResult() > 0;
		boolean supportsLimit = dialect.supportsLimit();
		boolean supportsOffset = dialect.supportsLimitOffset();
		if ((hasLimit || hasOffset) && supportsLimit)
		{
			sql = dialect.getLimitString(sql, getFirstResult(), getMaxResults());
			int insertIndex = dialect.bindLimitParametersFirst() ? 0 : parameterOrder.size();
			parameterOrder.add(insertIndex, "_limit_");
			setParameter("_limit_", getMaxResults() + (supportsOffset ? 0 : getFirstResult()), Type.INT);
			if (dialect.bindLimitParametersInReverseOrder()) insertIndex += 1;
			// reverse order = limit, offset
			if (hasOffset && supportsOffset) 
			{
				parameterOrder.add(insertIndex , "_offset_");
				setParameter("_offset_", getFirstResult(), Type.INT);
			}
		}
		return sql;
	}

	/**
		Set parameter values on a prepared statement.
		@param ps The prepared statement
		@param parameterOrder The names of the parameters in the order they
			appear in the prepared statement
	*/
	void setParameters(PreparedStatement ps, List<String> parameterOrder)
		throws SQLException
	{
		int i = 1;
		for (String name : parameterOrder)
		{
			Object parameterValue = getParameterValue(name);
			Type valueType = getParameterType(name);
			if (parameterValue instanceof Collection)
			{
				if (debugParamEnabled) logParam.debug("Binding collection to parameter index: "+i+"++");
				for (Object value : (Collection)parameterValue)
				{
					setParameterValue(ps, i++, value, valueType);
				}
			}
			else
			{
				setParameterValue(ps, i++, parameterValue, valueType);
			}
		}
	}
	
	/**
		Set a parameter value on a prepared statement. If the object is a
		{@link BasicItem} the id of the object is used a value.

		@param ps The prepared statement
		@param index The index of the parameter, starting at 1
		@param parameterValue The value of the parameter
	*/
	private void setParameterValue(PreparedStatement ps, int index, Object parameterValue, Type valueType)
		throws SQLException
	{
		if (parameterValue instanceof BasicItem)
		{
			int id = ((BasicItem)parameterValue).getId();
			if (debugParamEnabled) logParam.debug("Binding '"+id+"' to parameter index: "+index);
			ps.setInt(index, id);
		}
		/*
		else if (parameterValue instanceof Date)
		{
			// Postgres drives doesn't recognize dates in the 2-argument form
			if (debugParamEnabled) logParam.debug("Binding '"+parameterValue+"' to parameter index: "+index);
			ps.setObject(index, parameterValue, Types.DATE);
		}
		*/
		else
		{
			if (debugParamEnabled) logParam.debug("Binding '"+parameterValue+"' to parameter index: "+index);
			if (valueType == null)
			{
				ps.setObject(index, parameterValue);
			}
			else
			{
				ps.setObject(index, parameterValue, valueType.getSQLType());
			}
		}
	}
	
}
