/*
	$Id: Annotation.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.AnnotationData;
import net.sf.basedb.core.query.EntityQuery;

import java.util.List;
import java.util.Collections;

/**
	This class represents an annotation. Annotations are user-defined
	properties that can be attached to {@link Annotatable}
	items. They can be very useful in the filtering and analysis
	process.
	<p>
	An annotation must have an {@link AnnotationType},
	which controls the type of and how many values that can be attached 
	to an annotation.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class Annotation
	extends BasicItem<AnnotationData>
{
	/**
		The type of item represented by this class.
		@see Item#ANNOTATION
		@see #getType()
	*/
	public static final Item TYPE = Item.ANNOTATION;
	
	/**
		This filter gives everybody read permission to annotations.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = new QueryRuntimeFilterImpl();

	/**
		Create a new annotation.
		@param dc The DbControl object
		@param annotationType The annotation type of the new annotation
		@param annotationSet The annotation set of the new annotation
		@see AnnotationSet#getAnnotation(AnnotationType)
	*/
	static Annotation getNew(DbControl dc, AnnotationType annotationType, AnnotationSet annotationSet)
		throws BaseException
	{
		assert annotationSet != null : "annotationSet == null";
		assert annotationType != null : "annotationType == null";
		Annotation a = dc.newItem(Annotation.class, annotationSet);
		AnnotationData data = a.getData();
		data.setAnnotationSet(annotationSet.getData());
		data.setAnnotationType(annotationType.getData());
		data.setValues(annotationType.getValueType().newParameterValueData());
		return a;
	}
	
	/**
		Get an <code>Annotation</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>Annotation</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static Annotation getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Annotation a = dc.loadItem(Annotation.class, id);
		if (a == null) throw new ItemNotFoundException("Annotation[id="+id+"]");
		return a;
	}
	
	/**
		Get a query configured to retrieve annotations. This query may return items 
		which the logged in user doesn't have 
		read permission to. At the moment there is no way to solve this problem.
		@return An {@link ItemQuery} object
	*/
	static ItemQuery<Annotation> getQuery()
	{
		return new ItemQuery<Annotation>(Annotation.class, RUNTIME_FILTER);
	}
	
	private AnnotationSet annotationSet;

	Annotation(AnnotationData annotationData)
	{
		super(annotationData);
	}
	
	Annotation(AnnotationData annotationData, AnnotationSet annotationSet)
	{
		super(annotationData);	
		this.annotationSet = annotationSet;
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		READ permission is granted if the logged in user has READ permission on 
		the associated item. CREATE, WRITE and DELETE permissions are granted
		if the logged in user has WRITE permission on the associated item.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		AnnotationSet as = getAnnotationSet();
		if (as.hasPermission(Permission.WRITE))
		{
			granted |= Permission.grant(Permission.CREATE, Permission.READ, Permission.WRITE, Permission.DELETE);
		}
		else if (as.hasPermission(Permission.USE))
		{
			granted |= Permission.grant(Permission.USE);
		}
		else if (as.hasPermission(Permission.READ))
		{
			granted |= Permission.grant(Permission.READ);
		}
		super.initPermissions(granted, denied);
	}
	PluginPermission getPluginPermissions()
	{
		return PluginPermission.NO_PLUGIN;
	}
	// -------------------------------------------

	/**
		Get the annotation set this annotation belongs to.
		@return An <code>AnnotationSet</code>
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission for the annotation set (should never happen)
		@throws BaseException If anything else goes wrong when getting the annotation set.
	*/
	public AnnotationSet getAnnotationSet()
		throws PermissionDeniedException, BaseException
	{
		if (annotationSet == null)
		{
			return getDbControl().getItem(AnnotationSet.class, getData().getAnnotationSet());
		}
		else
		{
			return annotationSet;
		}		
	}

	/**
		Get the annotation type of this annotation.
		@return An <code>AnnotationType</code>
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission for the annotation type
	 * @throws BaseException If anything else goes wrong when getting the annotation type.
	*/
	public AnnotationType getAnnotationType()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(AnnotationType.class, getData().getAnnotationType());
	}
	
	
	/**
		Get the value type for this annotation type. It can't be change once
		the object has been created.
	  	@return Type value type as an enum. 
		@since 2.2
	*/
	public Type getValueType()
	{
		return Type.fromValue(getData().getAnnotationType().getValueType());
	}
	
	/**
		Get the values this annotation contains. The values are of a
		{@link Type} appropriate for the given annotation type, ie. Integer object
		for an integer annotation type. All objects in the list are of the same type.
		@see AnnotationType#getValueType()
		@return A list of objects
		@throws BaseException If there is an error
	*/
	public List<?> getValues()
		throws BaseException
	{
		List<?> values = Values.getItemValues(getDbControl(), getData().getValues().getValues());
		return Collections.unmodifiableList(values);
	}
	
	/**
		Get the version number of the item keeping the values. This is useful when you need
		to know if the annotation values has been modified by another transaction since
		they were read from the database. Unlike other items the version number of the
		annotation item doesn't change when the values are modified.
	  	@return The version number as an int. 
	*/
	public int getValuesVersion()
	{
		return getData().getValues().getVersion();
	}

	/**
		Set the value of the annotation, replacing any previous values.
		@param value The new value
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission for the annotation or read permission for the annotation
			type
		@throws InvalidDataException If the value isn't a valid value for the 
			annotation type, see {@link AnnotationType#validateAnnotationValue(Object)}
		@throws BaseException If there is another error
	*/
	public void setValue(Object value)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		AnnotationType annotationType = getAnnotationType();
		annotationType.validateAnnotationValue(value);
		getData().getValues().setSingleValue(Values.getDataValue(value));
	}

	/**
		Set the values of the annotation, replacing any previous values.
		@param values A list containing the new values
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission for the annotation or read permission for the annotation
			type
		@throws InvalidDataException If the number of values are too many according
			to the {@link AnnotationType#getMultiplicity()} setting any of the values 
			isn't a valid value for the annotation type, see 
			{@link AnnotationType#validateAnnotationValue(Object)}
		@throws BaseException If there is another error
	*/
	public void setValues(List<?> values)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.WRITE);
		AnnotationType annotationType = getAnnotationType();
		int multiplicity = annotationType.getMultiplicity();
		if (multiplicity > 0 && values.size() > multiplicity)
		{
			throw new InvalidDataException("Too many values. Max "+multiplicity+" value(s) are allowed for this annotation type");
		}
		for (Object value : values)
		{
			annotationType.validateAnnotationValue(value);
		}
		getData().getValues().replaceValues(Values.getDataValues(values));
	}

	private static class QueryRuntimeFilterImpl
		implements QueryRuntimeFilter
	{
		public void enableFilters(QueryRuntimeFilterManager manager, EntityQuery query, DbControl dc)
		{
			// Do not add any filters.
		}
	}
}

