/*
	$Id: BasicChildItem.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.BasicData;

/**
	An abstract base class for all items which are child items to
	a {@link BasicItem} parent item. This class implements the {@link 
	#initPermissions(int, int)} method and sets the permissions for the child
	item based on the permissions for type of the parent item.
	<p>
	If the logged in user has read/use permission for the parent,
	READ/USE permission is granted. If the logged in user has write
	permission for the parent, WRITE, DELETE and CREATE permission
	is granted.
	<p>
	This class should only be used if the parent item itself hasn't
	overridden the <code>initPermissions</code> method, since that method
	is never called.
	
	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
abstract class BasicChildItem<D extends BasicData>
	extends BasicItem<D>
{
	BasicChildItem(D data)
	{
		super(data);
	}

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		If the logged in user has read permission for the parent,
		read permission is granted. If the logged in user has the permission
		returned by the {@link #getPermissionForWriteDeleteAndCreate()} method
		for the parent, WRITE, DELETE and CREATE permission
		is granted.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		int parentPermission = 
			getSessionControl().getRolePermissions(getParentType());
		if (Permission.hasPermission(parentPermission, getPermissionForWriteDeleteAndCreate()))
		{
			granted |= Permission.grant(Permission.WRITE, Permission.DELETE, Permission.CREATE);
		}
		else if (Permission.hasPermission(parentPermission, Permission.USE))
		{
			granted |= Permission.grant(Permission.USE);
		}
		else if (Permission.hasPermission(parentPermission, Permission.READ))
		{
			granted |= Permission.grant(Permission.READ);
		}
		super.initPermissions(granted, denied);
	}
	PluginPermission getPluginPermissions()
	{
		return PluginPermission.NO_PLUGIN;
	}
	// -------------------------------------------
	
	/**
		Get the permission that is required on the parent item that gives
		write, delete and create permission on the child item. The defult 
		permission is write.
	*/
	Permission getPermissionForWriteDeleteAndCreate()
	{
		return Permission.WRITE;
	}
	
	/**
		Get the type of item that is the parent of this item.
		@return The item type of the parent item
	*/
	abstract Item getParentType();
	
}
