/*
	$Id: Config.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.Properties;
import java.io.InputStream;
import java.net.URL;

/**
	Use this class to access the configuration settings for BASE.
	<p>
	The configuration settings are always stored in a file named 
	<code>base.config</code>. It should be located at the root of 
	one of the paths in the CLASSPATH environment variable. For Tomcat this 
	means it should be located in the <code>WEB-INF/classes</code> folder.

	@author Nicklas
	@version 2.0
*/
public class Config
{
	/**
		Variable for holding the configuration properties.
	*/
	private static Properties config = null;

	private static boolean isInitialised = false;
	
	private static java.io.File configDir = null;
	
	/**
		Loads the settings from the configuration file.
		@throws ConfigurationException This exception is thrown if the
			configuration settings cannot be loaded
	*/
	static synchronized void init()
		throws ConfigurationException
	{
		if (isInitialised) return;
		try
		{
			URL baseConfig = Config.class.getResource("/base.config");
			configDir = new java.io.File(baseConfig.toURI()).getParentFile();
			InputStream is = baseConfig == null ? null : baseConfig.openStream();
			if (is == null)
			{
				throw new ConfigurationException("Can't find the properties file. Make sure '/base.config' is in the CLASSPATH.");
			}
			config = new Properties();
			config.load(is);
			is.close();
		}
		catch (Exception ex)
		{
			config = null;
			throw new ConfigurationException("Error while loading properties from '/base.config'. " + ex.getMessage());
		}
		isInitialised = true;
	}

	/**
		Unload all settings.
	*/
	static synchronized void unload()
	{
		isInitialised = false;
		config = null;
		configDir = null;
	}
	
	/**
		Get the directory where the BASE configuration files are located.
		@return The directory as a <code>java.io.File</code> object
		@since 2.6
	*/
	public static java.io.File getConfigDirectory()
	{
		return configDir;
	}
	
	/**
		Get the configuration setting specified by <code>key</code> as a String value.
		@param key The key for the setting
		@return The value specified by <code>key</code> or null if it is not found
	*/
	public static String getString(String key)
	{
		return getString(key, null);
	}

	/**
		Get the configuration setting specified by <code>key</code> as a String value.
		@param key The key for the setting
		@param defaultValue The value to return if the <code>key</code> is not found
		@return The value specified by <code>key</code> or <code>defaultValue</code> 
			if it is not found or is empty
	*/
	public static String getString(String key, String defaultValue)
	{
		if (!isInitialised) init();
		String value = config.getProperty(key, defaultValue);
		if (value != null) value = value.trim();
		return "".equals(value) ? defaultValue : value;
	}

	/**
		Get the configuration setting specified by <code>key</code> as an integer value.
		@param key The key for the setting
		@return The value specified by <code>key</code> or 0 if it is not found or
			is not a number
	*/
	public static int getInt(String key)
	{
		return getInt(key, 0);
	}

	/**
		Get the configuration setting specified by <code>key</code> as an integer value.

		@param key The key for the setting
		@param defaultValue The value to return if the <code>key</code> is not found
		@return The value specified by <code>key</code> or <code>defaultValue</code> 
			if it is not found
	*/
	public static int getInt(String key, int defaultValue)
	{
		if (!isInitialised) init();
		int result = defaultValue;
		String setting = config.getProperty(key);
		if (setting != null)
		{
			try
			{
				result = Integer.parseInt(setting);
			}
			catch (NumberFormatException ex)
			{}
		}
		return result;
	}
	
	/**
		Get the configuration setting specified by <code>key</code> as a long value.
	
		@param key The key for the setting
		@param defaultValue The value to return if the <code>key</code> is not found
		@return The value specified by <code>key</code> or <code>defaultValue</code> 
			if it is not found
	*/
	public static long getLong(String key, long defaultValue)
	{
		if (!isInitialised) init();
		long result = defaultValue;
		String setting = config.getProperty(key);
		if (setting != null)
		{
			try
			{
				result = Long.parseLong(setting);
			}
			catch (NumberFormatException ex)
			{}
		}
		return result;
	}

	/**
		Get the configuration setting specified by <code>key</code> as a boolean 
		value. FALSE is returned if the setting doesn't exist, if it contains an
		empty string, or one of the values 'no', 'false' or '0'.
		In all other cases TRUE is returned.
		@param key The key for the setting
		@return The value specified by <code>key</code>
	*/
	public static boolean getBoolean(String key)
	{
		if (!isInitialised) init();
		String setting = config.getProperty(key);
		return (setting == null
			|| setting.equals("")
			|| setting.equals("0")
			|| setting.equals("no")
			|| setting.equals("false")
			) ? false : true;
	}
	
	/**
		Get the default character set to use when no other has been 
		specified. It reads the setting from the <code>defaultCharset</code>
		setting and uses <code>ISO-8859-1</code> if not specified.
		@return The name of the default charset to use
		@since 2.1.1
	*/
	public static String getCharset()
	{
		return getString("defaultCharset", "ISO-8859-1");
	}


}
