/*
	$Id: DataCubeFilter.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.DataCubeFilterData;
import net.sf.basedb.core.data.SharedData;

/**
	This represents a filter in a datacube. A filter simply contains a
	set of column/position coordinates of spots that has passed som filtering
	criteria. In theory a filter can be used on any layer in the data cube, but
	in practice filters are not reused among bioassaysets with different layers.
	This item keeps track of two important things. The filter number
	and the number of bytes the data in this layer occupies on disk. 
	The number of bytes is reported up to the {@link Experiment} which is the
	item that handles user quota. 

	@author Nicklas
	@version 2.0
	@see DataCube
	@see FilterBatcher
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class DataCubeFilter
	extends ChildItem<DataCubeFilterData>
{
	/**
		The type of item represented by this class.
		@see Item#DATACUBEFILTER
		@see #getType()
	*/
	public static final Item TYPE = Item.DATACUBEFILTER;

	/**
		Create a new <code>DataCubeFilter</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param cube The cube the new filter is located in
		@return The new <code>DataCubeFilter</code> item
		@throws BaseException If there is an error
	*/
	static DataCubeFilter getNew(DbControl dc, DataCube cube)
		throws BaseException
	{
		DataCubeFilter filter = dc.newItem(DataCubeFilter.class);
		filter.setDataCube(cube);
		filter.setFilterNo(cube.getNextFilterNo());
		return filter;
	}

	/**
		Get a <code>DataCubeFilter</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>DataCubeFilter</code> item
		@throws ItemNotFoundException If an item with the specified id 
			is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static DataCubeFilter getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		DataCubeFilter filter = dc.loadItem(DataCubeFilter.class, id);
		if (filter == null) throw new ItemNotFoundException("DataCubeFilter[id=" + id + "]");
		return filter;
	}


	/**
	 	Creates a new data cube item from the given data.
		@param data the data
	*/
	DataCubeFilter(DataCubeFilterData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From BasicItem
		-------------------------------------------
	*/
	/**
		Delete data and update disk usage information on the 
		experiment if action == Action.DELETE
	*/
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			getDataCube().addBytes(-getBytes());
			getVirtualDb().deleteAllCubeFilterData(getDataCubeNo(), getFilterNo());
		}
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	/**
		Get the experiment.
	*/
	SharedData getSharedParent()
	{
		return getData().getDataCube().getVirtualDb().getExperiment();
	}
	/**
		USE permission is enough to be able to manage data cubes.
	*/
	Permission getPermissionForWriteDeleteAndCreate()
	{
		return Permission.USE;
	}
	// -------------------------------------------

	/**
		Get the data cube this filter belongs to.
		@return A <code>DataCube</code> object
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the item
		@throws BaseException If there is another error
	*/
	public DataCube getDataCube()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(DataCube.class, getData().getDataCube());
	}
	
	/**
		Set the data cube. This cannot be changed after the filter 
		has been created.
	*/
	private void setDataCube(DataCube cube)
	{
		getData().setDataCube(cube.getData());
	}

	/**
		Get the number of the data cube this filter belongs to.
	*/
	public short getDataCubeNo()
	{
		return getData().getDataCube().getCubeNo();
	}

	/**
		Get the number of this filter.
	*/
	public short getFilterNo()
	{
		return getData().getFilterNo();
	}
	private void setFilterNo(short filterNo)
	{
		getData().setFilterNo(filterNo);
	}
	
	/**
		Get the number of bytes the data stored in this filter occupies on disk.
	*/
	public long getBytes()
	{
		return getData().getBytes();
	}
	void addBytes(long bytes)
	{
		getData().setBytes(bytes + getData().getBytes());
		getDataCube().addBytes(bytes);
	}

	/**
		Get the virtual database this filter belongs to.
	*/
	VirtualDb getVirtualDb()
	{
		return getDbControl().getItem(VirtualDb.class, getData().getDataCube().getVirtualDb());
	}

}
