/*
	$Id: DynamicQuery.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Dynamic;

/**
	Represents a query against the dynamic database. A single
	query object may query against data in one data cube at a time. Most of
	the times however, this is limited to data in a single bioassayset.
	
	A query usually has the {@link VirtualTable#SPOT} or {@link VirtualTable#POSITION}
	as the root table for the query. Use factory methods in the {@link Dynamic} 
	class to join other tables and create restrictions based on the data in those tables.
	<p>
	This type of query is executed by the {@link #iterate(DbControl)} method
	and returns the result as a {@link DynamicResultIterator}.
	
	@author Nicklas
	@version 2.0
	@see BioAssaySet#getSpotData()
	@see BioAssaySet#getPositionData()
	@see ExtraValue#getValues()
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public abstract class DynamicQuery
	extends AbstractSqlQuery
{

	private final DataCube cube;
	private final VirtualDb virtualDb;
	private final VirtualTable rootTable;

	/**
		Create a new dynamic query.
		@param cube The cube to query against
		@param rootTable The root table of the query
	*/
	DynamicQuery(DataCube cube, VirtualTable rootTable)
	{
		super(rootTable.getQualifiedTableName(cube.getVirtualDb()));
		this.cube = cube;
		this.virtualDb = cube.getVirtualDb();
		this.rootTable = rootTable;
		restrictPermanent(
			new DynamicRestriction(VirtualColumn.CUBE, cube.getCubeNo())
		);

	}
	
	/*
		From the Query interface
		-------------------------------------------
	*/
	/**
		The alias of the item that is returned by this query.
		@see Item#getAlias()
	*/
	public String getRootAlias()
	{
		return HibernateUtil.quote(rootTable.getAlias());
	}
	// -------------------------------------------
	
	DataCube getDataCube()
	{
		return cube;
	}
	VirtualDb getVirtualDb()
	{
		return virtualDb;
	}
	VirtualTable getRootTable()
	{
		return rootTable;
	}
}
