/*
	$Id: QueryRuntimeFilterManager.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.util.Set;
import java.util.HashSet;

/**
	This class manages the enabling/disabling of {@link QueryRuntimeFilter}:s.
	Instances of this class are created by {@link AbstractEntityQuery} before
	a query is executed.

	@author Nicklas
	@version 2.0
	@see <a href="../../../../../development/overview/query/overview.html#filters">Query API overview - runtime filters</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
class QueryRuntimeFilterManager
{

	/**
		The names of already enabled filters.
	*/
	private final Set<String> enabledFilters;
	
	/**
		The Hibernate session where the filters should be enabled.
	*/
	private final org.hibernate.Session session;

	/**
		Create a new manager for the <code>DbControl</code>.
	*/
	QueryRuntimeFilterManager(DbControl dc)
	{
		this.enabledFilters = new HashSet<String>();
		this.session = dc.getHibernateSession();
	}
	
	/**
		Enable a filter, unless it has already been enabled
		@param filterName The name of the filter to enable
		@return A <code>org.hibernate.Filter</code> object, or null if the
			filter already was enabled
	*/
	org.hibernate.Filter enableFilter(String filterName)
	{
		org.hibernate.Filter filter = null;
		if (!enabledFilters.contains(filterName))
		{
			filter = HibernateUtil.enableFilter(session, filterName);
			enabledFilters.add(filterName);
		}
		return filter;
	}

	/**
		Disable a filter.
		@param filterName The name of the filter
	*/
	void disableFilter(String filterName)
	{
		if (enabledFilters.contains(filterName))
		{
			HibernateUtil.disableFilter(session, filterName);
			enabledFilters.remove(filterName);
		}
	}
	
	/**
		Disable all filters.
	*/
	void disableAll()
	{
		for (String filterName : enabledFilters)
		{
			HibernateUtil.disableFilter(session, filterName);
		}
		enabledFilters.clear();
	}
	
}
