/*
  $Id: VirtualTable.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import org.hibernate.Hibernate;
import org.hibernate.dialect.Dialect;

/**
	This enumeration defines constants that are used to specify different
	virtual tables in the dynamic part of the database used for analysis. 
	For each {@link VirtualDb} it is possible to create one real 
	table for each <code>VirtualTable</code> constant.
	<p>
	
	The information in these objects are used
	by {@link HibernateUtil#createVirtualTable(VirtualDb, VirtualTable)}
	to create the extra tables needed by an experiment to store analysed
	data. These objects are also used in query elements by {@link DynamicQuery}:s
	when it needs to reference a certain column in one of the dynamic tables.

	@author Nicklas
	@version 2.0
	@see VirtualColumn
	@see net.sf.basedb.core.query.Dynamic
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public enum VirtualTable
{

	/**
		Table that stores spot intensities for each spot in a data cube.
		The number of intensity columns are the same as the number of 
		channels in the experiment.
	*/
	SPOT("D#Spot", "spt",
		VirtualColumn.CUBE,
		VirtualColumn.LAYER,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION
	)
	{
		/**
			Return default columns plus one intesity column for each channel of 
			the raw data type.
		*/
		public VirtualColumn[] getColumns(RawDataType rdt)
		{
			VirtualColumn[] columns = super.getColumns(rdt);
			VirtualColumn[] allColumns = new VirtualColumn[columns.length + rdt.getChannels()];
			System.arraycopy(columns, 0, allColumns, 0, columns.length);
			for (int i = 0; i < rdt.getChannels(); ++i)
			{
				allColumns[i+columns.length] = VirtualColumn.channel(i+1);
			}
			return allColumns;
		}
	},
	
	/**
		Table that stores integer spot extra values. One value can
		be stored for each spot.
	*/
	SPOT_EXTRA_INT("D#SpotInt", "spi",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_INT
	),
	
	/**
		Table that stores float spot extra values. One value can
		be stored for each spot.
	*/
	SPOT_EXTRA_FLOAT("D#SpotFloat", "spf",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_FLOAT
	),
	
	/**
		Table that stores string spot extra values. One value can
		be stored for each spot.
	*/
	SPOT_EXTRA_STRING("D#SpotString", "sps",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_STRING
	),

	/**
		Table that maps position coordinates in a data cube to reporters.
		Each position may map to one reporter or null.
	*/
	POSITION("D#Pos", "pos",
		VirtualColumn.CUBE,
		VirtualColumn.POSITION,
		VirtualColumn.REPORTER_ID
	),
	
	/**
		Table that stores integer position extra values. One value can
		be stored for each position.
	*/
	POSITION_EXTRA_INT("D#PosInt", "psi",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_INT
	),
	
	/**
		Table that stores float position extra values. One value can
		be stored for each position.
	*/
	POSITION_EXTRA_FLOAT("D#PosFloat", "psf",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_FLOAT
	),
	
	/**
		Table that stores string position extra values. One value can
		be stored for each position.
	*/
	POSITION_EXTRA_STRING("D#PosString", "pss",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.POSITION,
		VirtualColumn.EXTRA_STRING
	),

	/**
		Table that maps columns coordinates in a data cube to raw
		bioassays. Each column may map to any number of raw
		bioassays. 
	*/
	COLUMN("D#Col", "col",
		VirtualColumn.CUBE,
		VirtualColumn.COLUMN,
		VirtualColumn.RAWBIOASSAY_ID
	),

	/**
		Table that stores integer column extra values. One value can
		be stored for each column.
	*/
	COLUMN_EXTRA_INT("D#ColInt", "cli",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.EXTRA_INT
	),
	
	/**
		Table that stores float column extra values. One value can
		be stored for each column.
	*/
	COLUMN_EXTRA_FLOAT("D#ColFloat", "clf",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.EXTRA_FLOAT
	),
	
	/**
		Table that stores string column extra values. One value can
		be stored for each column.
	*/
	COLUMN_EXTRA_STRING("D#ColString", "cls",
		VirtualColumn.CUBE,
		VirtualColumn.EXTRA,
		VirtualColumn.COLUMN,
		VirtualColumn.EXTRA_STRING
	),

	/**
		Table that maps column/position coordinates in a data cube to
		the id's of the raw spots their intensities where calculated from. Each 
		column/position coordinate may map to any number of raw spots.
	*/
	RAWPARENTS("D#RawParents", "rawmap",
		VirtualColumn.CUBE,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION,
		VirtualColumn.RAWDATA_ID
	),

	/**
		Table that holds column/position coordinates that remains after
		a filtering operation has been applied to a data cube layer.
	*/
	FILTER("D#Filter", "flt",
		VirtualColumn.CUBE,
		VirtualColumn.FILTER,
		VirtualColumn.COLUMN,
		VirtualColumn.POSITION
	);
	
	/**
		The name template, # is replaced by the ID of the {@link VirtualDb} item.
	*/
	private final String nameTemplate;
	/**
		The alias to use in queries for this table.
	*/
	private final String alias;
	
	/**
		The columns in the table.
	*/
	private final VirtualColumn[] columns;

	/**
	 	Creates a new virtual table.
	 	
		@param nameTemplate The template for generating table names
		@param alias The alias of the table used in queries
		@param columns The columns that are part of the table
	*/
	private VirtualTable(String nameTemplate, String alias, VirtualColumn... columns)
	{
		this.nameTemplate = nameTemplate;
		this.alias = alias;
		this.columns = columns;
	}
	
	/**
		Get the real table name of this table in the database.
		@param vdb The virtual database
	*/
	public String getTableName(VirtualDb vdb)
	{
		return nameTemplate.replace("#", Integer.toString(vdb.getId()));
	}
	
	/**
		Get the real table name of this table in the database, quoted
		and with catalog/schema information prepended to it.
		@param vdb The virtual database
	*/
	public String getQualifiedTableName(VirtualDb vdb)
	{
		Dialect dialect = HibernateUtil.getDialect();
		char oq = dialect.openQuote();
		char cq = dialect.closeQuote();
		String dynamicCatalog = Application.getDynamicCatalog();
		String dynamicSchema = Application.getDynamicSchema();
		StringBuilder tableName = new StringBuilder();
		if (dynamicCatalog != null) tableName.append(oq).append(dynamicCatalog).append(cq).append(".");
		if (dynamicSchema != null) tableName.append(oq).append(dynamicSchema).append(cq).append(".");
		tableName.append(oq).append(getTableName(vdb)).append(cq);
		return tableName.toString();
	}
	
	/**
		Get the alias of this table. Used in queries:
		<code>SELECT ... FROM table AS alias ... </code>
	*/
	public String getAlias()
	{
		return alias;
	}
	
	/**
		Get all columns in this table.
		@param rdt The raw data type of the experiment
	*/
	public VirtualColumn[] getColumns(RawDataType rdt)
	{
		return columns;
	}
	
	/**
		Check if the specified column is part of the virtual table or
		not. This method doesn't check columns that depends on the raw
		data type, ie. the number of intensity columns in the
		{@link #SPOT} table.
		
		@param column The column to check
		@return TRUE if the column is part of the table, FALSE otherwise
	*/
	public boolean hasColumn(VirtualColumn column)
	{
		for (VirtualColumn c : columns)
		{
			if (c == column) return true;	
		}
		return false;
	}
	
	/**
		Get the number of bytes a single row in the database occupies.
		The number doesn't include any variable length columns, ie. string
		columns. The caller should add the string length to the value
		returned from this method.
		@param rdt The raw data type of the experiment
	*/
	public long getBytesPerRow(RawDataType rdt)
	{
		long bytesPerRow = 0;
		for (VirtualColumn vc : getColumns(rdt))
		{
			org.hibernate.type.NullableType type = vc.getType();
			if (type == Hibernate.SHORT)
			{
				bytesPerRow += 2;
			}
			else if (type == Hibernate.INTEGER)
			{
				bytesPerRow += 4;
			}
			else if (type == Hibernate.FLOAT)
			{
				bytesPerRow += 4;
			}
			else if (type == Hibernate.STRING)
			{
				bytesPerRow += 0; // Actual string length is added by the batcher
			}
			else
			{
				throw new AssertionError("No bytes per row specified for type: "+type);
			}
		}
		return bytesPerRow;
	}
}
