/*
	$Id: ExtendableData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

/**
	This class is inherited by data classes that supports
	per-server additions of columns to the underlying database table.
	<p>
	The added columns must be described in the <code>extended-properties.xml</code>
	configuration file. The format of the XML file is forced by a DTD. In summary:

	<pre class="code">
&lt;?xml version="1.0" ?&gt;
&lt;!DOCTYPE extended-properties SYSTEM "extended-properties.dtd" &gt;
&lt;extended-properties&gt;
   &lt;class name="ReporterData"&gt;
      &lt;property
         name="extra1"
         title="Extra property"
         column="extra1"
         type="string"
         length="255"
         null="true"
         update="true"
         insert="true"
         averagemethod="geometric_mean"
         description="An extra property for all reporters"
      /&gt;
   &lt;/class&gt;
&lt;/extended-properties&gt;
</pre>

	<p>
	The <code>name</code> attribute of the <code>&lt;class&gt;</code> tag is the
	name of the data class that the extra property should be added to. That
	class must of course implement this interface. The <code>&lt;class&gt;</code>
	tag may contain one or more <code>&lt;property&gt;</code> tags. Here is a
	short description of the attributes for the <code>&lt;property&gt;</code> tag.

	<table border=1>
	<tr>
		<th>Attribute</th>
		<th>Description</th>
	</tr>
	<tr valign="top">
		<td>name (required)</td>
		<td>
			The property name of the extended property.
			See {@link net.sf.basedb.core.ExtendedProperty#getName()}. The name
			must only contain letters, numbers and underscores but the first character
			can't be a number. The name must be unique within the class.
		</td>
	</tr>
	<tr valign="top">
		<td>title</td>
		<td>
			The display title for the extended property.
			See {@link net.sf.basedb.core.ExtendedProperty#getTitle()}.
		</td>
	</tr>
	<tr valign="top">
		<td>description</td>
		<td>
			A short description of the extended property.
			See {@link net.sf.basedb.core.ExtendedProperty#getDescription()}.
		</td>
	</tr>
	<tr valign="top">
		<td>column (required)</td>
		<td>
			The database column name of the extended property. This value
			must be unique for each class. The value is validated by the 
			{@link net.sf.basedb.core.dbengine.DbEngine#isValidColumnName(String)}
			which normally means it must follow the same rules as the <code>name</code>
			attribute. See {@link net.sf.basedb.core.ExtendedProperty#getColumn()}.
		</td>
	</tr>
	<tr valign="top">
		<td>type (required)</td>
		<td>
			The type of the column. Allowed values are:
			<ul>
				<li>int
				<li>long
				<li>float
				<li>double
				<li>boolean
				<li>string
				<li>date
			</ul>
			See {@link net.sf.basedb.core.ExtendedProperty#getType()}.
		</td>
	</tr>
	<tr valign="top">
		<td>length</td>
		<td>
			The maximum allowed length of the value if it is a string property.
			See {@link net.sf.basedb.core.ExtendedProperty#getLength()}.
			The default value is 255.
		</td>
	</tr>
	<tr valign="top">
		<td>null</td>
		<td>
			If the column supports null values or not. Allowed
			values are:
			<ul>
			<li>true (default)
			<li>false
			</ul>
			See {@link net.sf.basedb.core.ExtendedProperty#isNullable()}.
		</td>
	</tr>
	<tr valign="top">
		<td>insert</td>
		<td>
			If the value for this property should be inserted into the database
			or not.
			<ul>
			<li>true (default)
			<li>false
			</ul>
			See {@link net.sf.basedb.core.ExtendedProperty#isInsertable()}.
		</td>
	</tr>
	<tr valign="top">
		<td>update</td>
		<td>
			If the value for this property should be updated in the database
			or not.
			<ul>
			<li>true (default)
			<li>false
			</ul>
			See {@link net.sf.basedb.core.ExtendedProperty#isUpdateable()}.
		</td>
	</tr>
	<tr valign="top">
		<td>averagable</td>
		<td>
			<b>This attribute has been deprecated and is replaced by <code>averagemethod</code>!</b>
			<br>
			If it makes sense to take the average of multiple values for this
			property. By default all numerical columns are averagable. For non-numerical
			columns, this attribute is ignored. The default method is to calculate the
			arithmetic mean.
			<ul>
			<li>true 
			<li>false
			</ul>
			See {@link net.sf.basedb.core.ExtendedProperty#isAveragable()}.
		</td>
	</tr>
	<tr valign="top">
		<td>averagemethod</td>
		<td>
			The method to use when calculating the average of a set of values.
			This attribute replaces the <code>averagable</code> attribute.
			The following values can be used:
			<ul>
			<li>none: do not use this property when calculating averages
			<li>arithmetic_mean: calculate the arithmetic mean
			<li>geometric_mean: calculate the geometric mean
			<li>min: use the minimum value among the values in the set
			<li>max: use the maximum value among the values in the set
			</ul>
			If no value is given to this attribute the default is to use arithmetic
			mean for all numerical value types.
			See {@link net.sf.basedb.core.ExtendedProperty#getAverageMethod()}.
		</td>
	</tr>
	</table>
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private Map&lt;String, Object&gt; extendedProperties;
public Object getExtended(String name)
{
   return extendedProperties == null ? null : extendedProperties.get(name);
}
public void setExtended(String name, Object value)
{
   if (extendedProperties == null) extendedProperties = new HashMap&lt;String, Object&gt;();
   extendedProperties.put(name, value);
}
</pre>

	@see net.sf.basedb.core.ExtendedProperties
	@see net.sf.basedb.core.ExtendedProperty
	@author Nicklas, Samuel
	@version 2.0
*/
public interface ExtendableData
	extends IdentifiableData
{
	/**
		Get the value of an extended property.
		@param name The name of the property
		@return The value of the property or null if not found
	*/
	public Object getExtended(String name);
	
	/**
		Set the value of an extended property.
		@param name The name of the property
		@param value The new value for the property
	*/
	public void setExtended(String name, Object value);

}
