/**
	$Id: PostgresDbEngine.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.core.dbengine;

import java.util.Set;

/**
	Database engine for Postgres.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class PostgresDbEngine
	extends AbstractDbEngine
	implements DbEngine
{

	/**
		Create PostgresDbEngine.
	*/
	public PostgresDbEngine()
	{}

	/*
		From the DbEngine interface
		-------------------------------------------
	*/
	/**
		Always false.
	*/
	public boolean useThetaJoin()
	{
		return false;
	}
	
	/**
		Generate <code>ALTER TABLE &lt;schema&gt;.&lt;table&gt; ADD CONSTRAINT &lt;name&gt;
		UNIQUE (&lt;columns&gt)</code> for a unique index, 
		<code>CREATE INDEX &lt;schema&gt;.&lt;name&gt; ON &lt;schema&gt;.&lt;table&gt; (&lt;columns&gt;)</code>
		for other indexes.
		The catalog parameter is not supported.
	*/
	public String getCreateIndexSql(String catalog, String schema, String table, String name, Set<String> columns, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		if (unique)
		{
			sql.append("ALTER TABLE ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" ADD CONSTRAINT ");
			sql.append(getQuotedName(name)).append(" UNIQUE (");
			int i = 0;
			for (String column : columns)
			{
				if (i > 0) sql.append(", ");
				sql.append(getQuotedName(column));
				i++;
			}
			sql.append(")");
		}
		else
		{
			sql.append("CREATE INDEX ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(name)).append(" ON ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" (");
			int i = 0;
			for (String column : columns)
			{
				if (i > 0) sql.append(", ");
				sql.append(getQuotedName(column));
				i++;
			}
			sql.append(")");
			
		}
		return sql.toString();
	}
	
	/**
		Generate <code>ALTER TABLE &lt;schema&gt;.&lt;table&gt; DROP CONSTRAINT &lt;name&gt;</code>
		for unique indexes, <code>DROP INDEX &lt;schema&gt;.&lt;name&gt;</code> for other indexes.
		The catalog parameter is not supported.
	*/
	public String getDropIndexSql(String catalog, String schema, String table, String name, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		if (unique)
		{
			sql.append("ALTER TABLE ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" DROP CONSTRAINT ");
			sql.append(getQuotedName(name));
		}
		else
		{
			sql.append("DROP INDEX ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(name));
		}
		return sql.toString();
	}
	
	/**
		Return true. Otherwise queries may take a very long time since
		indexes may not be used properly.
	*/
	public boolean analyzeAfterBatchOperation()
	{
		return true;
	}

	/**
		Generate <code>VACUUM FULL &lt;schema&gt;.&lt;table&gt;</code>.
		The catalog parameter is not supported.
	*/
	public String getOptimizeTableSql(String catalog, String schema, String table)
	{
		schema = schema == null ? "" : getQuotedName(schema) + ".";
		return "VACUUM FULL " + schema + getQuotedName(table);
	}
	
	/**
		Generate <code>ANALYZE &lt;schema&gt;.&lt;table&gt;</code>.
		The catalog parameter is not supported.
	*/
	public String getAnalyzeTableSql(String catalog, String schema, String table)
	{
		schema = schema == null ? "" : getQuotedName(schema) + ".";
		return "ANALYZE " + schema + getQuotedName(table);
	}
	/**
		Returns FALSE.
		@since 2.4
	*/
	public boolean caseInsensitiveComparison()
	{
		return false;
	}
	/**
		Returns FALSE.
	*/
	public boolean supportsColumnAliasInWhere()
	{
		return false;
	}
	/**
		Returns TRUE.
	*/
	public boolean supportColumnAliasInOrderBy()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	public boolean supportColumnAliasInGroupBy()
	{
		return true;
	}
	/**
		Returns FALSE.
	*/
	public boolean supportColumnAliasInHaving()
	{
		return false;
	}
	/**
		Returns TRUE.
	*/
	public boolean checkForInvalidNumberOperation()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	public boolean selectOrderByColumnsIfDistinct()
	{
		return true;
	}
	/**
		Return <code>&lt;value&gt; ~ &lt;regexp&gt;</code>.
		@since 2.8
	*/
	public String rlike(String value, String regexp)
	{
		return value + " ~ " + regexp;
	}
	// -------------------------------------------
	
	/**
		Put quotes (") around the name.
	*/
	private String getQuotedName(String name)
	{
		return "\"" + name + "\"";
	}

}
