/*
	$Id: HqlLeftJoin.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;

/**
	Left joins an association to another item: 
	<code>LEFT JOIN &lt;alias.property&gt; &lt;joinedAlias&gt;</code>
	to the query.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#HQL}.

	@author Nicklas
	@version 2.0
	@see Hql#leftJoin(String, String, String, Restriction, boolean)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class HqlLeftJoin
	implements Join
{
	private final String alias;
	private final String property;
	private final String joinedAlias;
	private final Restriction on;
	private final boolean fetch;

	HqlLeftJoin(String alias, String property, String joinedAlias, Restriction on, boolean fetch)
	{
		assert property != null : "property == null";
		assert joinedAlias != null : "joinedAlias == null";
		this.alias = alias;
		this.property = property;
		this.joinedAlias = joinedAlias;
		this.on = on;
		this.fetch = fetch;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.HQL)
		{
			return "LEFT JOIN " + (fetch && !query.isCounting() ? "FETCH " : "") +
				(alias == null ? query.getRootAlias() : alias) + "." + property + " " + joinedAlias +
				(on == null ? "" : " WITH " + on.toQl(query, dc));
		}
		else
		{
			throw new UnsupportedOperationException("'left join' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return The restriction to join on or null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return on == null ? null : Collections.singleton(on);
	}
	// -------------------------------------------
	/*
	 	From the Join interface
	 	-------------------------------------------
	*/
	public String getThetaJoin(Query query, DbControl dc) 
		throws BaseException 
	{
		throw new UnsupportedOperationException("DynamicJoin operation not supported by "+query.getQueryType()+" query.");
	}
		
	public String getThetaJoinCondition(Query query, DbControl dc) 
		throws BaseException 
	{
		throw new UnsupportedOperationException("DynamicJoin operation not supported by "+query.getQueryType()+" query.");
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "LEFT JOIN "+(alias == null ? "" : alias + ".") + property + " " + joinedAlias + 
			(on == null ? "" : " ON " + on.toString());
	}
	// -------------------------------------------
}

