/*
	$Id: AutoDetectFileFormat.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util;

import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Location;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.plugin.AutoDetectingImporter;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.InteractivePlugin;

import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Collections;
import java.io.InputStream;

/**
	This class has methods for autodetecting file formats.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class AutoDetectFileFormat
{

	/**
		Find all plugins with configurations which can import the specified file. 
		This method will check all plugins implementing the {@link
		AutoDetectingImporter} interface if they can import the specified file.
		
		@param dc The <code>DbControl</code> object to use for database access
		@param context A required context for the plugins (ie. what items to import)
		@param file The file to import
		@param pluginDef Check only the configurations of this plugin definition, or
			null if all plugin definitions should be checked
		@return A map which for each plugin definition contains a list
			of mathching configurations
		@throws InvalidDataException If the file parameter is null or if
			the file isn't located in {@link Location#PRIMARY} storage
		@throws BaseException If there is another error
	*/
	public static Map<PluginDefinition, List<PluginConfiguration>> findPlugins(DbControl dc, GuiContext context, File file, PluginDefinition pluginDef)
		throws InvalidDataException, BaseException
	{
		if (file == null) throw new InvalidUseOfNullException("file");
		if (file.getLocation() != Location.PRIMARY)
		{
			throw new InvalidDataException("File isn't in primary storage: " + file);
		}
		SessionControl sc = dc.getSessionControl();
		Object currentItem = null;
		if (context.getType() == GuiContext.Type.ITEM)
		{
			currentItem = context.getItem().getById(dc, sc.getCurrentContext(context.getItem()).getId());
		}

		Map<PluginDefinition, List<PluginConfiguration>> plugins = 
			new LinkedHashMap<PluginDefinition, List<PluginConfiguration>>();
		
		List<PluginDefinition> pluginDefs = null;
		if (pluginDef == null)
		{
			ItemQuery<PluginDefinition> pluginQuery = 
				PluginDefinition.getQuery(context, AutoDetectingImporter.class.getName());
			pluginQuery.order(Orders.asc(Hql.property("name")));
			pluginQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
			pluginDefs = pluginQuery.list(dc);
		}
		else
		{
			pluginDefs = Collections.singletonList(pluginDef);
		}
		
		for (PluginDefinition plugin : pluginDefs)
		{
			List<PluginConfiguration> configs = new LinkedList<PluginConfiguration>();
			if (!plugin.requiresConfiguration())
			{
				if (checkImportable(sc, plugin, null, file, context, currentItem))
				{
					configs.add(null);
				}
			}
			if (plugin.supportsConfigurations())
			{
				ItemQuery<PluginConfiguration> configQuery = plugin.getPluginConfigurations();
				configQuery.order(Orders.asc(Hql.property("name")));
				configQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
				for (PluginConfiguration config : configQuery.list(dc))
				{
					if (checkImportable(sc, plugin, config, file, context, currentItem))
					{
						configs.add(config);
					}
				}
			}
			if (configs.size() > 0)	plugins.put(plugin, configs);
		}
		return plugins;
	}
	
	private static boolean checkImportable(SessionControl sc, PluginDefinition plugin, 
		PluginConfiguration config, File file, GuiContext context, Object currentItem)
	{
		AutoDetectingImporter importer = null;
		boolean canBeUsed = false;
		try
		{
			importer = plugin.newInstance(AutoDetectingImporter.class, null, sc, config, null);
			boolean isInContext = true;
			if (plugin.isInteractive())
			{
				try
				{
					InteractivePlugin ip = (InteractivePlugin)importer;
					isInContext = ip.isInContext(context, currentItem) == null;
				}
				catch (Throwable t)
				{
					isInContext = false;
				}
			}
			if (isInContext)
			{
				InputStream in = file.getDownloadStream(0);
				try
				{
					canBeUsed = importer.isImportable(in);
				}
				catch (Throwable t)
				{
					canBeUsed = false;
				}
				finally
				{
					try
					{
						in.close();
					}
					catch (Throwable t)
					{}
				}
			}			
		}
		finally
		{
			if (importer != null) importer.done();
		}
		return canBeUsed;
	}
	
}
