/**
	$Id: ActionIterator.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions;

import java.util.Iterator;
import java.util.NoSuchElementException;


/**
	Iterator for iterating over all actions that <b>will be
	created</b> by extensions after a call to {@link
	Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}.
	That method returns an {@link ExtensionsInvoker} object which
	in it's turn uses this iterator.
	
	<p>
	Note the phrase "will be created" above. The actions 
	are created on demand as this iterator moves on the 
	next item. This happens in the {@link #hasNext()} method, which,
	for each extension, calls {@link Extension#getActionFactory()}
	and then {@link ActionFactory#getActions(InvokationContext)}.
	
	<p>
	The actions returned from the extensions are piled up in an
	internal buffer that the {@link #next()} method pick items from.
	When the buffer is empty the iterator moves on to the actions from
	next extension until there are no more extensions and the action
	buffer is empty.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ActionIterator<A extends Action>
	implements Iterator<A>
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.ActionIterator");

	// The current context
	private InvokationContext<A> currentContext;
	
	// An iterator over all usable extensions
	private final Iterator<InvokationContext<A>> iterator;
	
	// The actions created by the current extension
	private A[] actions;
	// The iterator's offset in the actions array
	private int offset;
	
	// Boolean flags to indicate if we have a next element and if 
	// we need to check or not. If both are 'false' we have reached the end
	private boolean checkNext;
	private boolean hasNext;
	
	
	ActionIterator(Iterator<InvokationContext<A>> iterator)
	{
		this.iterator = iterator;
		this.checkNext = true;
	}
	/*
		From the Iterator interface
		---------------------------
	*/
	public boolean hasNext()
	{
		if (checkNext)
		{
			checkNext = false;
			offset++;
			if (actions != null && offset < actions.length)
			{
				// There are more actions in the array
				hasNext = true;
			}
			else
			{
				// No more actions in the array, move on to the next extension
				currentContext = null;
				actions = null;
				offset = 0;
				hasNext = false;
				
				// Continue until we find an extension that creates at least one
				// action, or until we run out of extensions
				while (iterator.hasNext())
				{
					currentContext = iterator.next();
					
					// Get the actions for the extension
					actions = currentContext.getActions();

					// Validate that the actions are of the expected type
					if (validActions(actions))
					{
						hasNext = true;
						break;							
					}
				}
			}
		}
		return hasNext;
	}
	
	public A next()
	{
		if (checkNext) hasNext();
		if (!hasNext) throw new NoSuchElementException();
		checkNext = true;
		return actions[offset];
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
	// ------------------------------------
	
	/**
		Get the extension point that the current action
		belongs to.
	*/
	public ExtensionPoint<? super A> getExtensionPoint()
	{
		return currentContext.getExtensionPoint();
	}
	
	/**
		Get the extension that created the current action.
	*/
	public Extension<? extends A> getExtension()
	{
		return currentContext.getExtension();
	}
	
	/**
		Gets the renderer for the current action.
	*/
	Renderer<? super A> getRenderer()
	{
		return currentContext.getRenderer();
	}

	/**
		Check that the actions array is a not-null, non-empty array
		and contains actions of a type that is compatible with the
		{@link ExtensionPoint#getActionClass()} class.
		<p>
		NOTE! If the array contains at least one invalid action, all
		actions, including the valid ones, will be ignored. No exception
		will be thrown. An error-level log message is written to the log
		file.
		
		@param actions The actions to check
		@return TRUE if the actions are valid, FALSE otherwise
	*/
	private boolean validActions(A[] actions)
	{
		if (actions == null || actions.length == 0) return false;

		Class<?> actionClass = getExtensionPoint().getActionClass();
		for (int i = 0; i < actions.length; ++i)
		{
			if (!actionClass.isInstance(actions[i]))
			{
				Extension ext = getExtension();
				ClassCastException cc = new ClassCastException(actions[i] + " -> " + actionClass.getName());
				log.error("Action '" + actions[i] + "' created by extension '" + ext.getId() + 
					"' is not of the expected class '" + actionClass.getName(), cc);
				return false;
			}
		}
		return true;
	}

}