/**
	$Id: ExtensionContext.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions;

/**
	Invokation context for an extension. Most
	calls are forwarded to the invokation context for the
	extension point this extension belongs to.
	
	@author nicklas
	@version 2.7
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
 */
class ExtensionContext<A extends Action>
	extends InvokationContext<A>
{
	private final ExtensionPointContext<? super A> mainContext;
	private final Extension<A> extension;
	private Renderer<? super A> renderer;
	private boolean hasCreatedRenderer;
	
	ExtensionContext(ExtensionPointContext<? super A> mainContext, Extension<A> extension)
	{
		this.mainContext = mainContext;
		this.extension = extension;
	}
	/*
		From the InvokationContext object
		---------------------------------
	*/
	@Override
	public ClientContext getClientContext()
	{
		return mainContext.getClientContext();
	}

	@Override
	public ExtensionPoint<? super A> getExtensionPoint()
	{
		return mainContext.getExtensionPoint();
	}
	
	@Override
	public Extension<A> getExtension()
	{
		return extension;
	}
	
	@Override
	public Object getAttribute(String name)
	{
		return mainContext.getRegistry().getAttribute(extension.getId(), name);
	}

	@Override
	protected Registry getRegistry()
	{
		return mainContext.getRegistry();
	}
	
	@Override
	protected void prepareRendererFactory()
	{
		RendererFactory<? super A> factory = extension.getRendererFactory();
		if (factory != null) 
		{
			try
			{
				factory.prepareContext(this);
			}
			catch (Throwable t)
			{
				log.error("Error preparing renderer factory for extension '" +
					extension.getId() + "': " + factory, t);
			}
		}
	}
	
	/**
		If the extension point allows it and this extension
		specifies a renderer factory, create a new renderer from
		it. Otherwise, get the renderer from the extension point.
	*/
	@Override
	protected Renderer<? super A> getRenderer()
	{
		if (!hasCreatedRenderer)
		{
			ExtensionPoint<? super A> ep = mainContext.getExtensionPoint();
			if (ep.allowRendererOverride())
			{
				RendererFactory<? super A> factory = extension.getRendererFactory();
				if (factory != null)
				{
					renderer = factory.getRenderer(this);
				}
			}
			if (renderer == null)
			{
				renderer = mainContext.getRenderer();
			}
			hasCreatedRenderer = true;
		}
		return renderer;
	}

	/**
		Call {@link Extension#getActionFactory()} and then
		{@link ActionFactory#getActions(InvokationContext)} on the
		returned factory.
	*/
	@Override
	protected A[] getActions()
	{
		A[] actions = null;
		ActionFactory<? extends A> factory = extension.getActionFactory();
		try
		{
			actions = factory.getActions(this);
		}
		catch (Throwable t)
		{
			log.error("Error creating actions for extension '" +
					extension.getId() + "' using factory: " + factory, t);			
		}
		return actions;
	}
	// ----------------------------------------
	
	boolean prepareActionFactory()
	{
		boolean prepared = false;
		ActionFactory<? extends A> factory = extension.getActionFactory();
		try
		{
			prepared = factory.prepareContext(this);
		}
		catch (Throwable t)
		{
			log.error("Error preparing action factory for extension '" +
					extension.getId() + "': " + factory, t);
		}
		return prepared;
	}
	

}
