/**
	$Id: InvokationContext.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions;

/**
	Keeps contextual information about a single invokation of
	extensions. The {@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
	will create invokation contexts for each enabled extension that
	extends any of the given extension points.
	<p>
	
	Instances of this class are passed to factory methods,
	for example {@link ActionFactory#getActions(InvokationContext)}
	to make it possible for the factories to know anything about
	the context the extension was invoked in.
	<p>
	
	Do not confuse the invokation context with the a 
	{@link ClientContext} object. The invokation context contains
	the client context (see {@link #getClientContext()} but has also
	a lot more information collected from other parts of the 
	extensions system.

	@author nicklas
	@version 2.7
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public abstract class InvokationContext<A extends Action>
{
	protected static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.InvokationContext");
	
	protected InvokationContext()
	{}
	
	/**
		Get the client context that was passed to the 
		{@link Registry#useExtensions(ClientContext, ExtensionsFilter, String...)}
		method.
		@return The client context, or null if the client didn't pass
			any context object
	*/
	public abstract ClientContext getClientContext();

	/**
		Get information about the extension point.
		@return An extension point object
	*/
	public abstract ExtensionPoint<? super A> getExtensionPoint();
	
	/**
		Get information about the extension. This method may
		return null if there is no current extension, which only
		happend when any of the {@link RendererFactory} methods
		are called for a renderer factory attached to an extension
		point.
		@return An extension object, or null if there is no current extension
	*/
	public abstract Extension<A> getExtension();
	
	/**
		Get the value of an attribute that has been registered
		for the extension.
		
		@param name The name of the attribute
		@return The attribute value, or null if no attribute has
			been registered
		@see Registry#getAttribute(String, String)
	*/
	public abstract Object getAttribute(String name);

	/**
		Get the registry where this invokation originated from.
	*/
	protected abstract Registry getRegistry();
	
	/**
		Prepare the renderer factory if the extension or extension
		point has one.
	*/
	protected abstract void prepareRendererFactory();
	
	/**
		Get the renderer, if any. This method should always return the
		same rendere instance if called multiple times on the same object. 
	*/
	protected abstract Renderer<? super A> getRenderer();

	/**
		Get the actions, or null if there is no current extension.
	*/
	protected abstract A[] getActions();
	
}
