/*
	$Id: Log2Function.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.jep;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Expressions;

import java.util.Stack;

import org.nfunk.jep.Node;
import org.nfunk.jep.ParseException;


/**
	A JEP function class that adds a <code>log2(expression)</code> function to a 
	JEP expression parser. The function will look take the 2-based logarithm of
	it's argument.
	<p>
	This function is added automatically by the {@link Jep#newJep(String, JepFunction[])}
	method to the parser.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see Jep
*/
public class Log2Function
	implements JepExpressionFunction
{

	/**
		Get the singleton instance of this function.
		@return The Log2Function object
	 */
	public static Log2Function getInstance()
	{
		return theInstance;
	}
	
	/**
	 	The one and only instance of this function.
	*/
	private static final Log2Function theInstance = new Log2Function();
	
	/**
		The natural logarithm of 2. This is used when calculating the 2-based
		logarithm of a value since it can be calculated as: <code>ln(x) / ln(2)</code>
	*/
	private static final double LOG2 = Math.log(2);
	
	/**
		Create a new instance of this function.
	*/
	private Log2Function()
	{}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "log2"
	*/
	public String getFunctionName()
	{
		return "log2";
	}
	// -------------------------------------------
	/*
		From the JepExpressionFunction interface
		-------------------------------------------
	*/
	/**
		Use the {@link Expressions#log2(Expression)} method to create an
		expression taking the 2-based logarithm of the argument.
	*/
	public Expression toExpression(Node node)
	{
		int numChildren = node.jjtGetNumChildren();
		if (numChildren != 1)
		{
			throw new BaseException("Invalid number of arguments for 'log2' function: " + numChildren);
		}
		return Expressions.log2(Jep.nodeToExpression(node.jjtGetChild(0)));
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/
	/**
		@return Always 1
	*/
	public int getNumberOfParameters()
	{
		return 1;
	}
	public void setCurNumberOfParameters(int n)
	{}
	public boolean checkNumberOfParameters(int n)
	{
		return n == 1;
	}
	@SuppressWarnings({"unchecked"})
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null || stack.empty()) 
		{
			throw new ParseException("Stack is empty");
		}
		Object argument = stack.pop();
		if (argument instanceof Integer)
		{
			stack.push(log2((Number)argument));
		}
		else
		{
			throw new ParseException("Invalid parameter type: " + argument + "; expected number");
		}
	}
	// -------------------------------------------

	/**
		Calculate the 2-based logarithm of the argument.
	 	@param argument A number to calculate on
	 	@return a double object
	 	@throws ParseException If the logarithm could not be calculated.  
	*/
	public double log2(Number argument)
		throws ParseException
	{
		return Math.log(argument.doubleValue()) / LOG2;
	}
}
