/**
	$Id: PlateMappingExporter.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.plugins;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.PlateMapping;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.Job.ExecutionTime;
import net.sf.basedb.core.data.MappingCoordinate;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractExporterPlugin;
import net.sf.basedb.core.plugin.ExportOutputStream;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.Permissions;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;

/**
	Exporter for plate mappings. It has a fixed file format:

	<pre class="code">
Source plates=1
Source rows=3
Source columns=3
Destination plates=1
Destination rows=3
Destination columns=3
Source plate	Source row	Source column	Destination plate	Destination row	Destination column
0	0	0	0	0	0
0	0	1	0	1	0
0	0	2	0	2	0
0	1	0	0	0	1
0	1	1	0	1	1
0	1	2	0	2	1
0	2	0	0	0	2
0	2	1	0	1	2
0	2	2	0	2	2
</pre>

	 @author nicklas
	 @version 2.0
	 @base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class PlateMappingExporter
	extends AbstractExporterPlugin
	implements InteractivePlugin, SignalTarget
{

	private static final About about = 
		new AboutImpl
		(
			"Plate mapping exporter",
			"This plugin is used to export plate mappings, which maps the coordinates of one or more source plates " +
			"to coordinates on one or more destination plates.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2006, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
		);
	
	private static final Set<GuiContext> guiContexts = 
		Collections.singleton(new GuiContext(Item.PLATEMAPPING, GuiContext.Type.ITEM));
	
	private static final Set<Permissions> permissions = new HashSet<Permissions>();

 
	private PluginParameter<PlateMapping> plateMappingParameter;

	private RequestInformation configureJob;

	private ThreadSignalHandler signalHandler;

	/**
		Create a new importer.
	*/
	public PlateMappingExporter()
	{}

	/*
		From the Plugin interface
		-------------------------------------------
	*/
	public About getAbout()
	{
		return about;
	}
	public boolean supportsConfigurations()
	{
		return false;
	}
	public boolean requiresConfiguration()
	{
		return false;
	}
	/**
	 	Request create and write access to File:s and Directory:s and read access to
	 	PlateMapping:s and PlateGeometries.
	*/
	public Collection<Permissions> getPermissions()
	{
		if (permissions.size() == 0)
		{
			permissions.add(new Permissions(Item.FILE, null, EnumSet.of(Permission.CREATE, Permission.WRITE)));
			permissions.add(new Permissions(Item.DIRECTORY, null, EnumSet.of(Permission.CREATE, Permission.WRITE)));
			permissions.add(new Permissions(Item.USER, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.PLATEMAPPING, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.PLATEGEOMETRY, EnumSet.of(Permission.READ), null));
		}
		return permissions;
	}
	// -------------------------------------------

	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	/**
		Return a set containing the context:s [PLATEMAPPING, ITEM].
	*/
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	/**
		Always null.
	*/
	public String isInContext(GuiContext context, Object item)
	{
		String message = null;
		if (item == null)
		{
			message = "The object is null";
		}
		else if (!(item instanceof PlateMapping))
		{
			message = "The object is not a PlateMapping: " + item;
		}
		return message;
	}

	/**
		Store configuration settings for {@link Request#COMMAND_CONFIGURE_PLUGIN} and
		{@link Request#COMMAND_CONFIGURE_JOB}.
	*/
	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		try
		{
			if (command.equals(Request.COMMAND_CONFIGURE_JOB))
			{
				RequestInformation ri = getConfigureJobParameters(context, !request.isAllowedImmediateExecution());
				List<Throwable> errors = validateRequestParameters(ri.getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found in the request", errors);
					return;
				}
				
				storeValue(job, request, ri.getParameter("plateMapping"));
				if (request.getParameterValue(SAVE_AS) == null)
				{
					if (!request.isAllowedImmediateExecution())
					{
						response.setError("Immediate download is not allowed. Please specify a filename.", null);
						return;
					}
					PlateMapping pm = (PlateMapping)request.getParameterValue("plateMapping");
					response.setDownloadImmediately("Export plate mapping " + 
						pm.getName(), ExecutionTime.SHORTEST, true);
				}
				else
				{
					Object parameterValue = request.getParameterValue(OVERWRITE); 
					boolean overwrite = parameterValue != null ? (Boolean)parameterValue : false;
					if (!pathCanBeUsed((String)request.getParameterValue(ri.getParameter(SAVE_AS).getName()), overwrite))
					{
						response.setError("File exists: " + (String)request.getParameterValue(ri.getParameter(SAVE_AS).getName()), null);
						return;
					}
					storeValue(job, request, ri.getParameter(SAVE_AS));
					storeValue(job, request, ri.getParameter(OVERWRITE));
					response.setDone("The job configuration is complete", ExecutionTime.SHORTEST);
				}
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
	}
	public RequestInformation getRequestInformation(GuiContext context, String command)
		throws BaseException
	{
		RequestInformation requestInformation = null;
		if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestInformation = getConfigureJobParameters(context, null);
		}
		return requestInformation;
	}
	// -------------------------------------------
	
	/*
	 	From the AbstractExporterPlugin class
		-------------------------------------------
	*/
	private DbControl dc;
	private PlateMapping mapping;
	protected void begin(DbControl dc)
	{
		this.dc = dc;
		mapping = (PlateMapping)job.getValue("plateMapping");
		mapping = PlateMapping.getById(dc, mapping.getId());
	}
	protected void performExport(ExportOutputStream out, ProgressReporter progress)
		throws IOException
	{
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		
		PlateGeometry sourceGeometry = mapping.getSourceGeometry();
		PlateGeometry destGeometry = mapping.getDestinationGeometry();
		
		out.setMimeType("text/plain");
		String filename = Path.makeSafeFilename(mapping.getName(), "").toLowerCase()+".txt";
		out.setFilename(filename);
		
		// Wrap output stream in a writer
		Writer textOut = new OutputStreamWriter(out);
		// Headers
		textOut.write("Source plates="+mapping.getSourceCount()+"\n");
		textOut.write("Source rows="+sourceGeometry.getRows()+"\n");
		textOut.write("Source columns="+sourceGeometry.getColumns()+"\n");
		textOut.write("Destination plates="+mapping.getDestinationCount()+"\n");
		textOut.write("Destination rows="+destGeometry.getRows()+"\n");
		textOut.write("Destination columns="+destGeometry.getColumns()+"\n");
			
		// Data header
		textOut.write("Source plate\tSource row\tSource column\tDestination plate\tDestination row\tDestination column\n");
			
		// Mapping details
		for (int plate = 0; plate < mapping.getDestinationCount(); ++plate)
		{
			for (int row = 0; row < destGeometry.getRows(); ++row)
			{
				for (int column = 0; column < destGeometry.getColumns(); ++column)
				{
					checkInterrupted();
					MappingCoordinate destination = new MappingCoordinate(plate, row, column);
					MappingCoordinate source = mapping.getSourceCoordinate(destination);
					if (source != null)
					{
						textOut.write(source.getPlate() + "\t" + source.getRow() + "\t" + source.getColumn() + "\t");
						textOut.write(destination.getPlate() + "\t" + destination.getRow() + "\t" + destination.getColumn() + "\n");
					}
				}
			}
		}
		textOut.flush();
	}
	protected void end(boolean success)
	{
		this.dc = null;
	}
	protected String getSuccessMessage()
	{
		return "Plate mapping " + mapping.getName() + " exported successfully";
	}
	// -------------------------------------------
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------

	private RequestInformation getConfigureJobParameters(GuiContext context, Boolean requireFile)
	{
		if (configureJob == null)
		{
			// RequestInformation for CONFIGURE_JOB
			
			if (requireFile == null)
			{
				PluginDefinition pd = job.getPluginDefinition();
				requireFile = pd == null ? false : !pd.getAllowImmediateExecution();
			}
			String defaultPath = null;
			if (requireFile)
			{
				int currentMappingId = sc.getCurrentContext(Item.PLATEMAPPING).getId();
				if (currentMappingId != 0)
				{
					DbControl dc = sc.newDbControl();
					try
					{
						PlateMapping pm = PlateMapping.getById(dc, currentMappingId);
						defaultPath = "~/" + Path.makeSafeFilename(pm.getName(), "").toLowerCase() + ".txt";
					}
					finally
					{
						if (dc != null) dc.close();
					}
				}
			}
			
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			plateMappingParameter = new PluginParameter<PlateMapping>(
				"plateMapping", 
				"Plate mapping", 
				"The plate mapping to export", 
				new ItemParameterType<PlateMapping>(PlateMapping.class, null, true, 1, null)
			);

			parameters.add(plateMappingParameter);
			parameters.add(getSaveAsParameter(null, null, defaultPath, requireFile));
			parameters.add(getOverwriteParameter(null, null));
			
			configureJob = new RequestInformation
			(
				Request.COMMAND_CONFIGURE_JOB,
				"Select a place to save the exported data.",
				"Here you select where to save the exported data and how to handle existing files.",
				parameters
			);
		}
		return configureJob;
	}

	
}
