/**
	$Id: HybridizationImporter.java 4574 2008-10-13 07:43:43Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.ArraySlide;
import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.LabeledExtract;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

/**
	Plug-in for importing hybridization items in a batch. The plug-in can create new 
	items and updated existing items.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-10-13 09:43:43 +0200 (Mon, 13 Oct 2008) $
*/
public class HybridizationImporter
	extends AbstractItemImporter<Hybridization>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.HYBRIDIZATION, GuiContext.Type.LIST));

	private static final About about =
		new AboutImpl
	(
			"Hybridization importer",
			"Imports and updates hybridizations in a batch.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2008, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
	);

	
	protected static final PluginParameter<String> hardwareColumnMapping = new PluginParameter<String>(
			"hardwareColumnMapping",
			"Hybridization station",
			"Mapping that picks the name or ID of a hybridization station from the data columns. " +
			"The plug-in will first try to find a station with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Hyb. station\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> arraySlideColumnMapping = new PluginParameter<String>(
			"arraySlideColumnMapping",
			"Array slide",
			"Mapping that picks the name or ID of an array slide from the data columns. " +
			"The plug-in will first try to find a slide with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Array slide\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> numArraysColumnMapping = 
		new PluginParameter<String>(
				"numArraysColumnMapping",
				"Number of arrays",
				"Mapping that picks the number of arrays that fits on a single slide. The " +
				"values must be in the numeric format specified by the 'Number format' parameter." +
				"Example: \\Num arrays\\",
				optionalColumnMapping
		);

	protected static final PluginParameter<String> arrayIndexColumnMapping = 
		new PluginParameter<String>(
				"arrayIndexColumnMapping",
				"Array index",
				"Mapping that picks the (1-based) index of the array a specific labeled extract " +
				"was used on. The value should be between 1 and the number of arrays and must be " +
				"in the numeric format specified by the 'Number format' parameter." +
				"Example: \\Array index\\",
				optionalColumnMapping
		);
	
	public HybridizationImporter()
	{}

	/*
		From the Plugin interface
		------------------------------------
	*/
	@Override
	public About getAbout()
	{
		return about;
	}
	// ------------------------------------

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper numArraysMapper;
	private Mapper createdMapper;
	private Mapper protocolMapper;
	private Mapper hardwareMapper;
	private Mapper parentMapper;
	private Mapper usedQuantityMapper;
	private Mapper arrayIndexMapper;
	private Mapper arraySlideMapper;
	
	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME,  
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	/**
		Adds column mappings for name, externalId, description,
		protocol, creation date, original quantity, pooled, parent
		and used quantity from parent.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(numArraysColumnMapping);
		parameters.add(protocolColumnMapping);
		parameters.add(hardwareColumnMapping);
		
		parameters.add(SampleImporter.createdColumnMapping);
		parameters.add(SampleImporter.parentColumnMapping);
		parameters.add(SampleImporter.usedQuantityColumnMapping);
		parameters.add(arrayIndexColumnMapping);
		parameters.add(arraySlideColumnMapping);
	}
	
	/**
		Calls {@link Hybridization#getQuery()}.
	*/
	@Override
	protected ItemQuery<Hybridization> createItemQuery()
	{
		return Hybridization.getQuery();
	}
	
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? Hybridization.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? Hybridization.MAX_DESCRIPTION_LENGTH : null, null);
		numArraysMapper = getMapper(ffp, (String)job.getValue("numArraysColumnMapping"), null, null);
		createdMapper = getMapper(ffp, (String)job.getValue("createdColumnMapping"), null, null);
		protocolMapper = getMapper(ffp, (String)job.getValue("protocolColumnMapping"), null, null);
		hardwareMapper = getMapper(ffp, (String)job.getValue("hardwareColumnMapping"), null, null);
		parentMapper = getMapper(ffp, (String)job.getValue("parentColumnMapping"), null, null);
		usedQuantityMapper = getMapper(ffp, (String)job.getValue("usedQuantityColumnMapping"), null, null);
		arrayIndexMapper = getMapper(ffp, (String)job.getValue("arrayIndexColumnMapping"), null, null);
		arraySlideMapper = getMapper(ffp, (String)job.getValue("arraySlideColumnMapping"), null, null);
	}
	
	@Override
	protected Hybridization createItem(DbControl dc, FlatFileParser.Data data)
	{
		Hybridization hyb = Hybridization.getNew(dc);
		updateItem(dc, hyb, data);
		return hyb;
	}
	
	@Override
	protected void updateItem(DbControl dc, Hybridization hyb, FlatFileParser.Data data)
	{
		if (nameMapper != null) hyb.setName(nameMapper.getValue(data));
		if (descriptionMapper != null) hyb.setDescription(descriptionMapper.getValue(data));
		if (numArraysMapper != null) hyb.setNumArrays(numArraysMapper.getInt(data));
		BioMaterialEvent creationEvent = hyb.getCreationEvent();
		if (createdMapper != null) creationEvent.setEventDate(parseDate(createdMapper.getValue(data)));
		if (protocolMapper != null) 
		{
			String nameOrId = protocolMapper.getValue(data);
			Protocol protocol = findProtocol(dc, FallbackIdMethod.NAME_OR_ID, nameOrId);
			if (nameOrId == null || protocol != null) creationEvent.setProtocol(protocol);
		}
		if (hardwareMapper != null)
		{
			String nameOrId = hardwareMapper.getValue(data);
			Hardware hybStation = findHybridizationStation(dc, FallbackIdMethod.NAME_OR_ID, nameOrId);
			if (nameOrId == null || hybStation != null) creationEvent.setHardware(hybStation);
		}
		if (arraySlideMapper != null)
		{
			String nameOrId = arraySlideMapper.getValue(data);
			ArraySlide slide = findArraySlide(dc, FallbackIdMethod.NAME_OR_BARCODE_OR_ID, nameOrId);
			if (slide != null && slide.isUsedByOther(hyb))
			{
				// NOTE! There is currently no error handling option to bypass this
				// It will be implemented in 2.9
				throw new PermissionDeniedException(Permission.USE, 
					slide + ". It is already used by another hybridization.");
			}
			if (nameOrId == null || slide != null) hyb.setArraySlide(slide);
		}
		updateMultiLineItem(dc, hyb, data, 0);
	}
	
	/**
		Add parent labeled extracts and the used quantities
	*/
	@Override
	protected void updateMultiLineItem(DbControl dc, Hybridization hyb, 
		FlatFileParser.Data data, int multiLineNum)
	{
		if (parentMapper != null)
		{
			String nameOrId = parentMapper.getValue(data);
			LabeledExtract parent = findLabeledExtract(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId);
			if (parent != null)
			{
				BioMaterialEvent creationEvent = hyb.getCreationEvent();
				Float usedQuantity = usedQuantityMapper == null ?
						null : usedQuantityMapper.getFloat(data);
				creationEvent.addSource(parent, usedQuantity);
				if (arrayIndexMapper != null)
				{
					Integer arrayIndex = arrayIndexMapper.getInt(data);
					if (arrayIndex != null) creationEvent.setSourceGroup(parent, arrayIndex);
				}
			}
		}
	}
	// ------------------------------------
}
