import java.io.InputStream;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

import net.sf.basedb.util.FileUtil;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;
import net.sf.basedb.util.parser.FlatFileParser.Line;

/*
 $Id$

 Copyright (C) 2005 Johan Enell, Nicklas Nordborg
 Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
 Copyright (C) 2007 Johan Enell
 
 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 */

/**
 
 @base.modified $Date$
 @author Enell
 @version 2.0
 */
public class TestFlatFileParser
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing flat file parser");

		FlatFileParser ffp = test_create("\"(.*)=(.*)\"", "\"?(.*?)\"?(\\t\"?(.*?)\"?){10,}", null, "\\t", null, null, 20, 10, 0);

		test_parse(ffp, "data/test.parse.hskg.plates.txt", "\\3\\", "\\cluster_id\\", "Row:\\1\\, Col:\\2\\");
		test_parse(ffp, "data/test.parse.genepix.result.gpr", "\\3\\", "\\Name\\", "B\\Block\\R\\Row\\C\\Column\\", "=col('Dia.')",
			"=col('Dia.') / 2", "=\\Name\\+':'+\\ID\\");

		ffp = test_create("(.*)=,(.*)", null, null, ",", null, "\\[(.*)\\]", 20, 10, 0);
		test_parse_section(ffp, "data/test.parse.printmap.mwbr");

		write("++Testing file parser " + (ok ? "OK" : "Failed") + "\n");
		return ok;
	}

	public static FlatFileParser test_create(String header, String data_header, String data_footer, String data, String ignore, String section, int max_unknown_lines, int min_data_columns, int max_data_columns)
	{
		FlatFileParser ffp = new FlatFileParser();
		if (header != null) ffp.setHeaderRegexp(Pattern.compile(header));
		if (data_header != null) ffp.setDataHeaderRegexp(Pattern.compile(data_header));
		if (data_footer != null) ffp.setDataFooterRegexp(Pattern.compile(data_footer));
		if (data != null) ffp.setDataSplitterRegexp(Pattern.compile(data));
		ffp.setTrimQuotes(true);
		if (ignore != null) ffp.setIgnoreRegexp(Pattern.compile(ignore));
		if (section != null) ffp.setSectionRegexp(Pattern.compile(section));
		ffp.setMaxUnknownLines(max_unknown_lines);
		ffp.setMinDataColumns(min_data_columns);
		ffp.setMaxDataColumns(max_data_columns);
		return ffp;
	}

	public static void test_parse(FlatFileParser ffp, String file, String... mappings)
	{
		try
		{
			InputStream in = FileUtil.getInputStream(new java.io.File(file));
			ffp.setInputStream(in, "ISO-8859-1");
			FlatFileParser.LineType last_line_type = ffp.parseHeaders();
			int i = 0;
			int headers = ffp.getLineCount();
			while (i < headers)
			{
				write(i, ffp.getLine(i));
				i++;
			}
			if (!TestUtil.getSilent()) write("--Column headers: " + ffp.getColumnHeaders());
			// write("--Last line: "+types[last_line_type]);
			if ((last_line_type == FlatFileParser.LineType.DATA_HEADER) || (last_line_type == FlatFileParser.LineType.DATA))
			{
				Mapper[] mappers = new Mapper[mappings.length];
				for (int j = 0; j < mappings.length; ++j)
				{
					mappers[j] = ffp.getMapper(mappings[j]);
				}
				while (ffp.hasMoreData() && i < 20 + headers)
				{
					write(i++, ffp.nextData(), mappers);
				}
			}
			in.close();
			write("--Parse OK (" + file + ")");
		}
		catch (Throwable ex)
		{
			write("--Parse FAILED (" + file + ")");
			ex.printStackTrace();
			ok = false;
		}
	}

	public static void test_parse_section(FlatFileParser ffp, String file)
	{
		try
		{
			Set<String> sections = new HashSet<String>(Arrays.asList("FileInformation", "Block1", "Block2", "Block3", "Block4", "mapping")); 
			InputStream in = FileUtil.getInputStream(new java.io.File(file));
			ffp.setInputStream(in, "ISO-8859-1");
			FlatFileParser.Line section;
			while (ffp.hasMoreSections())
			{
				section = ffp.nextSection();
				sections.remove(section.name());
				ffp.parseHeaders();
				ffp.nextData();
			}
			in.close();
			if (!sections.isEmpty())
			{
				throw new Exception("Didn't find all sections in (" + file + "), missed: " + sections);
			}
			write("--Parse sections OK (" + file + ")");
		}
		catch (Throwable ex)
		{
			write("--Parse sections FAILED (" + file + ")");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void write(String message)
	{
		System.out.println(message);
	}

	static void write(int i, FlatFileParser.Line line)
	{
		if (!TestUtil.getSilent())
		{
			if (line.type() == FlatFileParser.LineType.HEADER)
			{
				write("Line " + i + ": \t" + line.type() + "\t{" + line.name() + "} -> {" + line.value() + "}");
			}
			else
			{
				String s = line.line();
				if (s.length() > 30) s = s.substring(0, 30);
				write("Line " + i + ": \t" + line.type() + "\t" + s);
			}
		}
	}

	static void write(int i, FlatFileParser.Data data, Mapper[] mappings)
	{
		if (!TestUtil.getSilent())
		{
			StringBuffer sb = new StringBuffer();
			sb.append("Line " + i + ": \tDATA(" + data.columns() + ")\t");
			int max = data.columns();
			if (max > 5) max = 5;
			for (int j = 0; j < max; j++)
			{
				sb.append(j + "={" + data.get(j) + "}\t");
			}
			if (mappings != null)
			{
				for (Mapper m : mappings)
				{
					sb.append("M{" + m.getValue(data) + "}\t");
				}
			}
			write(sb.toString());
		}
	}

	public static void testHeaderLine(Line line, String name, String value) throws Exception
	{
		if (!line.type().equals(FlatFileParser.LineType.HEADER))
			throw new Exception("Line is no header: "+line.toString());
		if (!line.name().equals(name))
			throw new Exception("Cant find name '"+name+"' in line '"+line.line()+"'");
		if (!line.value().equals(value))
			throw new Exception("Cant find value '"+value+"' in line '"+line.line()+"'");
	}

	public static void testLine(Line line, String name) throws Exception
	{
		if (!line.line().equals(name))
			throw new Exception("Line '"+line.line()+"' doesnt match '"+name+"'");
	}

	public static void testSection(Line line, String name) throws Exception
	{
		if (!line.type().equals(FlatFileParser.LineType.SECTION))
			throw new Exception("Line is no section: "+line.toString());
		if (!line.name().equals(name))
			throw new Exception("Cant find sectionname '"+name+"' in line '"+line.line()+"'");
	}
}
