/**
	$Id: CmdLine.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.test;

import java.util.HashMap;
import java.util.Map;

/**
	Utility class for parsing command line arguments. It supports a very limitied
	syntax: [options] [cmd]
	<p>
	Options starts with hyphen (-) and may have a value following it. The last
	parameter is the command unless it starts with a hyphen.
	<p>
	Examples: 
	<pre class="code">
./jobagent.sh start
./jobagent.sh -c agent.properties stop
</pre>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class CmdLine
{
	private final Map<String, String> options;
	private final String cmd;
	
	/**
		Create a new object for parsing the command line.
		@param args The command line arguments sent to the <code>main()</code> method
	*/
	public CmdLine(String[] args)
	{
		this.options = new HashMap<String, String>();
		String currentOption = null;
		
		// Cmd is the last argument
		int lastIndex = args.length-1;
		if (lastIndex >= 0 && !args[lastIndex].startsWith("-"))
		{
			this.cmd = args[lastIndex];
		}
		else
		{
			this.cmd = null;
		}
		
		for (int i = 0; i <= lastIndex; ++i)
		{
			String arg = args[i];
			if (arg.startsWith("-"))
			{
				options.put(arg, null);
				currentOption = arg;
			}
			else if (currentOption != null)
			{
				options.put(currentOption, arg);
				currentOption = null;
			}
		}
	}
	
	/**
		Get the command parameter
		@return The command parameter, or null
	*/
	public String getCmd()
	{
		return cmd;
	}
	
	/**
		Get the value for the specified option
		@param option The option to get the value for
		@return The value or null
	*/
	public String getOption(String option)
	{
		return options.get(option);
	}
	
	/**
		Get the value for an option.
		@param option The option to get the value for
		@param defaultValue A default value if the option wasn't specified
		@return The options value or the default value
	*/
	public String getOption(String option, String defaultValue)
	{
		return options.containsKey(option) ? options.get(option) : defaultValue;
	}
	
	/**
	 	Check if an option was specified or not.
		@param option The option to check
		@return TRUE if the option was specified, FALSE otherwise
	*/
	public boolean hasOption(String option)
	{
		return options.containsKey(option);
	}
	
}
