/**
	$Id: RawDataTest.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.performance;

import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Import raw data.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class RawDataTest
{

	public static boolean ok = true;

	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password, int numIterations)
	{
		DbControl dc = null;
		TestUtil.write("++Entering raw data test\n");
		try
		{
			TestUtil.login(user, password, false);

			// Create raw bioassays
			dc = TestUtil.getDbControl();
			Platform generic = Platform.getByExternalId(dc, Platform.GENERIC);
			RawDataType genepix = RawDataTypes.getSafeRawDataType("genepix");
			
			// Load prepared items
			ArrayDesign design = PrepareTest.getArrayDesign(dc);
			Experiment experiment = PrepareTest.getExperiment(dc);
			PluginConfiguration rawDataImporter = PrepareTest.getRawDataImporter(dc);
			File rawData1 = PrepareTest.getRawDataFile1(dc);
			File rawData2 = PrepareTest.getRawDataFile2(dc);
			
			long startNum = experiment.getRawBioAssays().count(dc);
			RawBioAssay[] rba = new RawBioAssay[numIterations];
			for (int i = 0; i < numIterations; ++i)
			{
				rba[i] = createRawBioAssay(dc, generic, genepix, "PerformanceTest #" + (startNum + i), design);
			}
			dc.commit();
			
			// Import raw data
			long minTime = Long.MAX_VALUE;
			long maxTime = 0;
			long totalTime = 0;
			for (int i = 0; i < numIterations; i += 2)
			{
				TestUtil.gc();
				long time = importRawData(rawDataImporter, rawData1, rba[i]) / 1000;
				totalTime += time;
				if (time < minTime) minTime = time;
				if (time > maxTime) maxTime = time;
				TestUtil.write("Import #" + i + "\t" + Values.formatTime(time) + "\n");

				time = importRawData(rawDataImporter, rawData2, rba[i+1]) / 1000;
				totalTime += time;
				if (time < minTime) minTime = time;
				if (time > maxTime) maxTime = time;
				TestUtil.write("Import #" + (i + 1) + "\t" + Values.formatTime(time) + " s\n");
			}
			TestUtil.write("---------------------------------------------\n");
			TestUtil.write("Imported " + numIterations + " raw data sets\n");
			TestUtil.write("Max time: " + Values.formatTime(maxTime) + "\n");
			TestUtil.write("Min time: " + Values.formatTime(minTime) + "\n");
			TestUtil.write("Mean time: " + Values.formatTime(totalTime / numIterations) + "\n");
			TestUtil.write("---------------------------------------------\n");
			
			// Add raw bioassays to experiment
			dc = TestUtil.getDbControl();
			dc.reattachItem(experiment);
			for (int i = 0; i < numIterations; i++)
			{
				dc.refreshItem(rba[i]);
				experiment.addRawBioAssay(rba[i]);
			}
			dc.commit();
			
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving raw data test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a raw bioassay.
	*/
	public static RawBioAssay createRawBioAssay(DbControl dc, Platform platform, RawDataType rawDataType, String name, ArrayDesign design)
	{
		TestUtil.write("--Creating raw bioassay: " + name + "\n");
		RawBioAssay rba = RawBioAssay.getNew(dc, platform, rawDataType);
		rba.setName(name);
		rba.setArrayDesign(design);
		dc.saveItem(rba);
		return rba;
	}

	/**
		Import raw data.
	*/
	public static long importRawData(PluginConfiguration config, File file, RawBioAssay rba)
		throws Exception
	{
		TestUtil.write("--Importing raw data from file: " + file.getName() + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			config = PluginConfiguration.getById(dc, config.getId());
			Job job = Job.getNew(dc, config.getPluginDefinition(), config, null);
			job.setName("PerformanceTest: Import data to raw bioassay: " + rba.getName());
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(true);
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("rawBioAssay", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null), rba);
			job.setParameterValue("missingReporterError", new StringParameterType(), "skip");
			job.setParameterValue("featureMismatchError", new StringParameterType(), "skip");
			dc.saveItem(job);
			dc.commit();
			long time = System.currentTimeMillis();
			PluginUtil.executeJob(job);
			return  System.currentTimeMillis() - time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
