/**
	$Id: ArrayDesignClient.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.ws.client;

import net.sf.basedb.info.ArrayDesignInfo;
import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.QueryOptions;

import java.io.IOException;
import java.io.InputStream;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
  	A web service client for array design in BASE.
  	This class supports all services in 
  	{@link net.sf.basedb.ws.server.ArrayDesignService}
  	@author Martin
 	@version 2.5
 */
public class ArrayDesignClient 
	extends AbstractRPCClient
{

	private SessionClient session = null;
	
	/**
	 	Creates a new client object for a session.
	  	@param session
	 */
	public ArrayDesignClient(SessionClient session)
	{
		super(session.getUrl(), "ArrayDesign", session.getServiceFactory());
		this.session = session;
	}

	/**
		Get info about an array design when you know the ID.
		@param itemId The ID of the array design
		@return An ArrayDesignInfo object
		@throws AxisFault If communication with web service fails.
	*/
	public ArrayDesignInfo getById(int itemId)
		throws AxisFault
	{
		return invokeBlocking("getById", ArrayDesignInfo.class, 
			session.getId(), itemId);
	}
	
	/**
	 	Gets info about array designs that are available in the logged in session.
	  	@param qOpt Options to put on the query
	  	@return An array with info object for each array design that was found. 
	  	@throws AxisFault If communication with web service fails.
	 */
	public ArrayDesignInfo[] getArrayDesigns(QueryOptions qOpt)
		throws AxisFault
	{
		ArrayDesignInfo[] result = invokeBlocking("getArrayDesigns", ArrayDesignInfo[].class, session.getId(), qOpt);
		return result;
	}
	
	/**
	 	Downloads an arraydesign as a file, 
	 	if it is stored as a file in BASE.
	  	@param arrayDesignId The array design's id.
	  	@param dataFileType The external ID of the data file type to download
	  	@return An InputStream to read the file data from, or null
	  		if no file of the specified type exists
	  	@throws IOException If writing to the file fails.
	*/
	public InputStream downloadArrayDesignByType(int arrayDesignId, String dataFileType)
		throws IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadArrayDesignByType", 
				session.getId(), 
				arrayDesignId, 
				dataFileType
			);
	}
	
	/**
	 	Gets the datafile types that is used to store an array design.
	  	@param arrayDesignId The array design's id
	  	@param qOpt Restrictions and includes to put on a query when getting the types.
	  	@return An array with a DataFileTypeInfo for each found data file type.
	  	@throws AxisFault If communication with the webservice fails.
	 */
	public DataFileTypeInfo[] getDataFileTypes(int arrayDesignId, QueryOptions qOpt) 
		throws AxisFault
	{
		return invokeBlocking
			(
				"getDataFileTypes", 
				DataFileTypeInfo[].class, 
				session.getId(), 
				arrayDesignId, 
				qOpt
			);
	}
	
	/**
	 	Gets if data in an array design can be downloaded as a file or not.
	  	@param arrayDesignId Id of the array design to check
	  	@return TRUE if there are any data to download as a file, FALSE otherwise.
	  	@throws AxisFault If communication with the webservice fails.
	 */
	public Boolean hasDownloadableData(int arrayDesignId) 
		throws AxisFault
	{
		return invokeBlocking
			(
				"hasDownloadableData",
				Boolean.class,
				session.getId(),
				arrayDesignId
			);
	}
}
